Package: L1centrality
Title: Graph/Network Analysis Based on L1 Centrality
Version: 0.5.2
Authors@R: c(
  person("Seungwoo", "Kang", email = "kangsw@skku.edu", role = c("aut", "cre"), 
  comment=c(ORCID="0000-0001-8082-0794")),
  person("Hee-Seok", "Oh", email = "heeseok@stats.snu.ac.kr", role = c("aut")))
Description: Analyze graph/network data using L1 centrality and prestige. Functions for deriving global, local, and group L1 centrality/prestige are provided. Routines for visual inspection of a graph/network are also provided. Details are in Kang and Oh (2026a) <doi:10.1080/01621459.2025.2520467>, Kang and Oh (2026b) <doi:10.1080/00031305.2025.2563730>, and Kang (2025) <doi:10.23170/snu.000000188358.11032.0001856>.
URL: https://github.com/seungwoo-stat/L1centrality
BugReports: https://github.com/seungwoo-stat/L1centrality/issues
License: GPL (>= 3)
Encoding: UTF-8
Depends: R (>= 2.10)
LazyData: true
Imports: doParallel, foreach, graphics, igraph, methods, parallel,
        Rcpp, stats, utils, withr
LinkingTo: Rcpp
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-07-14 01:41:52 UTC; seungwookang
Author: Seungwoo Kang [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8082-0794>),
  Hee-Seok Oh [aut]
Maintainer: Seungwoo Kang <kangsw@skku.edu>
Repository: CRAN
Date/Publication: 2026-07-14 02:40:12 UTC
Built: R 4.4.3; x86_64-apple-darwin20; 2026-07-14 03:56:03 UTC; unix
Archs: L1centrality.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(L1cent,igraph)
S3method(L1cent,matrix)
S3method(L1centEDGE,igraph)
S3method(L1centEDGE,matrix)
S3method(L1centGROUP,igraph)
S3method(L1centGROUP,matrix)
S3method(L1centLOC,igraph)
S3method(L1centLOC,matrix)
S3method(L1centMDS,igraph)
S3method(L1centMDS,matrix)
S3method(L1centNB,igraph)
S3method(L1centNB,matrix)
S3method(group_reduce,igraph)
S3method(group_reduce,matrix)
S3method(plot,L1cent)
S3method(plot,L1centEDGE)
S3method(plot,L1centLOC)
S3method(plot,L1centMDS)
S3method(print,L1cent)
S3method(print,L1centEDGE)
S3method(print,L1centGROUP)
S3method(print,L1centLOC)
S3method(print,L1centMDS)
S3method(print,L1centNB)
S3method(summary,L1cent)
S3method(summary,L1centEDGE)
S3method(summary,L1centLOC)
S3method(summary,L1centNB)
export(Gini)
export(L1cent)
export(L1centEDGE)
export(L1centGROUP)
export(L1centLOC)
export(L1centMDS)
export(L1centNB)
export(Lorenz_plot)
export(group_reduce)
importFrom(Rcpp,sourceCpp)
importFrom(foreach,"%dopar%")
importFrom(graphics,lines)
importFrom(graphics,text)
importFrom(igraph,distances)
importFrom(stats,aggregate)
importFrom(stats,cmdscale)
importFrom(stats,dist)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(utils,flush.console)
importFrom(withr,local_par)
useDynLib(L1centrality, .registration = TRUE)

#' @name Heterogeneity
#' @title Lorenz Curve and the Gini Coefficient
#'
#' @description
#' Draws a Lorenz curve (the group heterogeneity plot) and computes the Gini
#' coefficient (the group heterogeneity index).
#'
#' @param x A numeric vector.
#' @param add A logical value.
#'  * \code{TRUE}: add the Lorenz curve to an already existing plot.
#'  * \code{FALSE} (the default): draw the Lorenz curve to a new graphic device.
#' @param ... Further graphical parameters supplied to the internal
#'   [base::plot()] (when \code{add = FALSE}) or [graphics::lines()] (when
#'   \code{add = TRUE}) function. See [graphics::par()] document.
#' @return `Lorenz_plot()` draws a Lorenz curve (the group heterogeneity plot) and returns an
#'   invisible copy of a Gini coefficient (the group heterogeneity index).
#'
#'   `Gini()` returns a Gini coefficient.
#'
#' @export
#' @seealso \code{plot()} methods for objects of class [L1cent] and [L1centLOC]
#'   support plotting a Lorenz curve. \code{summary()} methods for objects of
#'   class [L1cent], [L1centLOC], and [L1centNB] provide the Gini coefficient as
#'   one of the summary statistics.
#'
#' @examples
#' vertex_weight <- igraph::V(MCUmovie)$worldwidegross
#' cent <- L1cent(MCUmovie, vertex_weight = vertex_weight)
#' gini <- Lorenz_plot(cent, asp = 1) # one can use "plot(cent, asp = 1)"
#' graphics::abline(0, 1, lty = 2)
#' # group heterogeneity index
#' gini
#' gini == Gini(cent)
#' @references S. Kang and H.-S. Oh. On a notion of graph centrality based on
#'   \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} data depth.
#'   \emph{Journal of the American Statistical Association}, 121(553): 400--412, 2026.
#'
#'   M. O. Lorenz. Methods of measuring the concentration of wealth.
#'   \emph{Publications of the American Statistical Association}, 9(70): 209--219, 1905.
Lorenz_plot <- function(x, add = FALSE, ...){
  x <- sort(x)
  mu <- mean(x)
  n <- length(x)
  Fx <- c(0,seq(1/n, 1, length.out = n))
  Deltax <- c(0,1/mu*cumsum(x)/n)
  if(!add){
    plot(Fx, Deltax, type="l", xlab="p", ylab="L(p)", ...)
  }else{
    graphics::lines(Fx, Deltax, ...)
  }
  invisible(unname(1 - 2*(Fx[2])*(sum(Deltax)-1/2))) # Gini index
}


#' @name Heterogeneity
#' @aliases Gini
#' @export
Gini <- function(x){
  x <- sort(x)
  mu <- mean(x)
  n <- length(x)
  Fx <- c(0,seq(1/n, 1, length.out = n))
  Deltax <- c(0,1/mu*cumsum(x)/n)

  unname(1 - 2*(Fx[2])*(sum(Deltax)-1/2))
}

#' @name L1cent
#' @aliases summary.L1cent
#' @param object An \code{L1cent} object, obtained as a result of the function
#'   \code{L1cent()}.
#' @return `summary.L1cent()` returns an object of class `table`.
#'   It is a summary of the prominence values with the five-number summary,
#'   mean, and the Gini coefficient.
#' @export
summary.L1cent <- function(object, ...){
  args <- list(...)
  structure(c(summary.default(object, ...),
              Gini = if(is.null(args$digits)) Gini(object) else signif(Gini(object), digits = args$digits)),
            class = "table")
}

#' @name L1centLOC
#' @aliases summary.L1centLOC
#' @param object An \code{L1centLOC} object, obtained as a result of the function
#'   \code{L1centLOC()}.
#' @return `summary.L1centLOC()` returns an object of class `table`.
#'   It is a summary of the prominence values with the five-number summary,
#'   mean, and the Gini coefficient, at each level of `alpha`.
#' @export
summary.L1centLOC <- function(object, ...){
  summat <- as.matrix(t(sapply(object, summary.L1cent, ...)))
  rownames(summat) <- round(attr(object, "alpha"), 4)
  names(dimnames(summat)) <- c("alpha", "")
  structure(summat, class = "table")
}

#' @name L1centNB
#' @aliases summary.L1centNB
#' @param object An \code{L1centNB} object, obtained as a result of the function
#'   \code{L1centNB()}.
#' @return `summary.L1centNB()` returns an object of class `table`.
#'   It is a summary of the prominence values with the five-number summary,
#'   mean, and the Gini coefficient, at each modified graph.
#' @export
summary.L1centNB <- function(object, ...){
  summat <- as.matrix(t(sapply(object, summary.L1cent, ...)))
  rownames(summat) <- names(object)
  names(dimnames(summat)) <- c("Modified w.r.t.", "")
  structure(summat, class = "table")
}

#' @name L1centEDGE
#' @aliases summary.L1centEDGE
#' @param object An \code{L1centEDGE} object, obtained as a result of the function
#'   \code{L1centEDGE()}.
#' @return `summary.L1centEDGE()` returns an object of class `table` with the
#'   number of local medians at each locality level `alpha`.
#' @export
summary.L1centEDGE <- function(object, ...){
  summat <- as.matrix(sapply(object, function(mat) length(unique(mat[,2]))))
  colnames(summat) <- "Number of local medians"
  names(dimnames(summat)) <- c("alpha","")
  structure(summat, class = "table")
}

#' @name L1centLOC
#' @aliases L1presLOC
#' @title Local L1 Centrality/Prestige
#'
#' @description
#' Computes local \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
#' centrality or local \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
#' prestige at each \code{alpha} level for every vertex. For undirected graphs,
#' the two measures are identical.
#'
#' @note
#' The function is valid only for connected graphs. If the graph is directed, it
#' must be strongly connected.
#'
#' @details
#' Suppose that the given graph has \ifelse{html}{\out{<i>n</i>}}{\eqn{n}}
#' vertices. We choose about \eqn{n\alpha} vertices
#' (\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality- or
#' \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige-based
#' neighborhood) for each vertex (see [L1centNB()]), and compute the
#' \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality or
#' \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige of the vertex
#' conditioned on these vertices, i.e., derive the
#' \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality or
#' \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige locally. For
#' details, refer to Kang and Oh (2026a) for undirected graphs, and Kang and Oh
#' (2026b) for directed graphs.
#'
#' @inheritParams L1cent
#' @param alpha A number or a numeric vector of locality levels. Values
#'   must be between 0 and 1.
#' @param parallel A boolean indicating whether to run the algorithm
#'   parallel across multiple cores. By default set to \code{FALSE}.
#' @return \code{L1centLOC()} returns an object of class \code{L1centLOC}. It is
#'   a list of numeric vectors. The length of the list is equivalent to the
#'   length of \code{alpha}, and the names of the list are the values of
#'   \code{alpha}. Each component of the list is a numeric vector whose length
#'   is equivalent to the number of vertices in the graph \code{g}.
#'   Specifically, the \code{i}th component of the list is a vector of local
#'   \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality at level
#'   \code{alpha[i]} for each vertex (if \code{mode = "centrality"}) or local
#'   \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige at level
#'   \code{alpha[i]} for each vertex (if \code{mode = "prestige"}).
#'
#'  `print.L1centLOC()` prints local
#'  \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality or local
#'  \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige values at
#'  each locality level `alpha` and returns them invisibly.
#'
#' @export
#' @seealso [L1cent()] for
#'   \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality/prestige,
#'   [L1centNB()] for \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
#'   centrality/prestige-based neighborhood.
#'
#' @examples
#' weight <- igraph::V(MCUmovie)$worldwidegross
#' MCUmovie_cent <- L1cent(MCUmovie, vertex_weight = weight)
#' MCUmovie_loc_cent <- L1centLOC(MCUmovie, vertex_weight = weight, alpha = 5/32)
#' plot(MCUmovie_cent, MCUmovie_loc_cent,
#'      main = "MCU movie network: global vs. local centrality")
#' @references S. Kang and H.-S. Oh. On a notion of graph centrality based on
#'   \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} data depth.
#'   \emph{Journal of the American Statistical Association}, 121(553): 400--412, 2026a.
#'
#'   S. Kang and H.-S. Oh.
#'   \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prominence measures
#'   for directed graphs. \emph{The American Statistician}, 80(2): 301--309, 2026b.
L1centLOC <- function(g, vertex_weight, alpha, mode, edge_weight_transform, parallel) UseMethod("L1centLOC")

#' @name L1centLOC
#' @exportS3Method L1centLOC igraph
L1centLOC.igraph <- function(g, vertex_weight = NULL, alpha, mode = c("centrality", "prestige"), edge_weight_transform = NULL, parallel = FALSE){
  validate_igraph(g, checkdir = FALSE)

  new_weight <- edge_weight_transform0(g, edge_weight_transform)
  if(!is.null(new_weight)) igraph::E(g)$weight <- new_weight
  D <- igraph::distances(g, mode = "out")
  attr(D, "label.igraph") <- igraph::V(g)$label
  L1centLOC.matrix(D, vertex_weight = vertex_weight, alpha, mode, parallel = parallel)
}

#' @name L1centLOC
#' @exportS3Method L1centLOC matrix
L1centLOC.matrix <- function(g, vertex_weight = NULL, alpha, mode = c("centrality", "prestige"), edge_weight_transform = NULL, parallel = FALSE){
  eta <- vertex_weight
  if(is.null(eta)) eta <- rep(1,ncol(g))
  validate_matrix(g, eta, checkdir = FALSE)
  if(!all(alpha >= 0 & alpha <= 1))
    stop("alpha is not in the correct range: [0,1]")
  mode <- match.arg(tolower(mode), choices = c("centrality", "prestige"))
  label <- rownames(g)
  if(is.null(label)) label <- 1:nrow(g)

  calls <- sys.calls()
  fnames <- sapply(calls, function(cl) as.character(cl[[1]]))
  # print(fnames)
  if(all(fnames[1:2] == c("L1centLOC", "L1centLOC.matrix")))
    message("DISTANCE matrix received")

  if(identical(alpha, 1) | identical(alpha, 1L)){
    loc.cent <- list(c(L1cent(g, eta, mode)))
    names(loc.cent) <- alpha
    return(structure(loc.cent,
                     class = "L1centLOC",
                     mode = mode,
                     alpha = alpha,
                     label = label,
                     label.igraph = attr(g, "label.igraph")))
  }

  if(is.null(rownames(g))) rownames(g) <- colnames(g) <- 1:ncol(g)

  n <- ncol(g)
  sg <- ifelse(isSymmetric.matrix(g), 1,
               min((g/t(g))[upper.tri(g) | lower.tri(g)]))
  # m <- ceiling(n*alpha)
  # label <- colnames(g)
  NB <- L1centNB(g, vertex_weight = eta, mode)
  loc.cent <- vector("list", length = length(alpha))
  # for (i in seq_along(alpha)) {
  #   nb.index <-
  #     lapply(NB, function(l)
  #       which(l >= stats::quantile(l, 1 - alpha[i], type=1)))
  #   loc.cent[[i]] <-
  #     sapply(1:length(nb.index), function(j){
  #       index <- which(rownames(g.new <- g[nb.index[[j]], nb.index[[j]]]) == names(nb.index)[j])
  #       if(identical(mode, "centrality")) g.new <- t(g.new)
  #       closenessinv <- colSums((eta.new <- eta[nb.index[[j]]])*g.new)
  #       1 - sg*max((closenessinv[index] - closenessinv)/(g.new + diag(Inf,nrow(g.new)))[index,]/sum(eta.new))
  #     })
  #   names(loc.cent[[i]]) <- rownames(g)
  # }
  if(parallel & length(alpha) >= 2){
    numCores <- min(parallel::detectCores()-2, length(alpha))
    cl <- parallel::makeCluster(numCores)
    doParallel::registerDoParallel(cl)
    loc.cent <- foreach::foreach(i = seq_along(alpha), .multicombine = TRUE, .combine='list') %dopar% {
      nb.index <-
        lapply(NB, function(l)
          which(l >= stats::quantile(l, 1 - alpha[i], type=1)))
      res <-
        sapply(1:length(nb.index), function(j){
          index <- which(rownames(g.new <- g[nb.index[[j]], nb.index[[j]]]) == names(nb.index)[j])
          if(identical(mode, "centrality")) g.new <- t(g.new)
          closenessinv <- colSums((eta.new <- eta[nb.index[[j]]])*g.new)
          1 - sg*max((closenessinv[index] - closenessinv)/(g.new + diag(Inf,nrow(g.new)))[index,]/sum(eta.new))
        })
      names(res) <- rownames(g)
      return(res)
    }
    parallel::stopCluster(cl)
  }else{
    for (i in seq_along(alpha)) {
      nb.index <-
        lapply(NB, function(l)
          which(l >= stats::quantile(l, 1 - alpha[i], type=1)))
      loc.cent[[i]] <-
        sapply(1:length(nb.index), function(j){
          index <- which(rownames(g.new <- g[nb.index[[j]], nb.index[[j]]]) == names(nb.index)[j])
          if(identical(mode, "centrality")) g.new <- t(g.new)
          closenessinv <- colSums((eta.new <- eta[nb.index[[j]]])*g.new)
          1 - sg*max((closenessinv[index] - closenessinv)/(g.new + diag(Inf,nrow(g.new)))[index,]/sum(eta.new))
        })
      names(loc.cent[[i]]) <- rownames(g)
    }
  }
  names(loc.cent) <- alpha
  return(structure(loc.cent,
                   class = "L1centLOC",
                   mode = mode,
                   alpha = alpha,
                   label = label,
                   label.igraph = attr(g, "label.igraph")))
}

#' @name L1centLOC
#' @aliases print.L1centLOC
#'
#' @param x An \code{L1centLOC} object, obtained as a result of the function
#'   \code{L1centLOC()}.
#' @param ... Further arguments passed to or from other methods.
#' @export
print.L1centLOC <- function(x, ...){
  for(i in seq_along(x)){
    cat("local L1 ", attr(x, "mode"), " at alpha = ",
        round(attr(x, "alpha")[[i]], 4), ":\n", sep = "")
    print.default(c(x[[i]]), ...)
    if(i != length(x)) cat("\n")
  }
  return(invisible(x))
}

#' @name L1centLOC
#' @aliases plot.L1centLOC
#'
#' @param y An optional argument providing the coordinates for a scatter plot.
#'   It could be an object of class \code{L1cent} or \code{L1centLOC}, or a
#'   numeric vector.
#' @param add A logical value. This argument is considered only when drawing a Lorenz curve.
#'  * \code{TRUE}: add the Lorenz curve to an already existing plot.
#'  * \code{FALSE} (the default): draw the Lorenz curve to a new graphic device.
#' @param threshold A number between 0 and 1. Vertices that have their maximum
#'   and minimum local \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
#'   prominence value difference above the \code{threshold} are indicated in
#'   colored lines.
#' @return `plot.L1centLOC()` draws a following plot.
#'  * \code{y} is not supplied and \code{alpha} is of length one: A Lorenz curve (the group heterogeneity plot)
#'  and returns an invisible copy of a Gini coefficient (the group heterogeneity
#'  index). \code{threshold} is ignored.
#'  * \code{y} is supplied and \code{alpha} is of length one: A scatter plot of
#'  \code{x} versus \code{y}. \code{threshold} is ignored.
#'  * \code{alpha}'s length is larger than one: A plot of \code{alpha} versus
#'  local \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prominence
#'  values (in a uniform margin) for each vertex. If \code{threshold} is set,
#'  vertices that have their maximum
#'   and minimum local \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
#'   prominence value difference above the \code{threshold} are indicated in
#'   colored lines. \code{y} is ignored.
#' @export
plot.L1centLOC <- function(x, y = NULL, add = FALSE, threshold = NULL, ...){
  args <- list(...)
  if(length(attr(x, "alpha")) == 1){
    if(is.null(y)){
      args$add <- add
      if(is.null(args$main)) args$main <- "Lorenz plot"
      do.call(Lorenz_plot, c(list(unlist(x)), args))
    }else{
      if(methods::is(y,"L1centLOC")){
        if(is.null(args$xlab)) args$xlab <- as.expression(bquote("Local" ~ L[1] ~ .(attr(x, "mode")) ~ "(alpha =" ~ .(attr(x, "alpha"))*")"))
        if(is.null(args$ylab)) args$ylab <- as.expression(bquote("Local" ~ L[1] ~ .(attr(y, "mode")) ~ "(alpha =" ~ .(attr(y, "alpha"))*")"))
        do.call(plot, c(list(as.numeric(unlist(x)), as.numeric(unlist(y))), args))
      }else if(methods::is(y,"L1cent")){
        if(is.null(args$xlab)) args$xlab <- as.expression(bquote("Local" ~ L[1] ~ .(attr(x, "mode")) ~ "(alpha =" ~ .(attr(x, "alpha"))*")"))
        if(is.null(args$ylab)) args$ylab <- as.expression(bquote(L[1] ~ .(attr(y, "mode"))))
        do.call(plot, c(list(as.numeric(unlist(x)), as.numeric(y)), args))
      }else{
        if(is.null(args$xlab)) args$xlab <- as.expression("Local" ~ bquote(L[1] ~ .(attr(x, "mode")) ~ "(alpha =" ~ .(attr(x, "alpha"))*")"))
        if(is.null(args$ylab)) args$ylab <- "y"
        do.call(plot, c(list(as.numeric(x), y), args))
      }
    }
  }else{
    x_unif <- sapply(x, function(vec) stats::ecdf(vec)(vec))
    x_unif <- x_unif[,order(attr(x, "alpha"))]
    attr(x, "alpha") <- sort(attr(x, "alpha"))
    if(is.null(threshold)){
      if(is.null(args$xlim)) args$xlim <- range(attr(x, "alpha"))
      if(is.null(args$ylim)) args$ylim <- c(0, 1)
      if(is.null(args$xlab)) args$xlab <- "Locality level"
      if(is.null(args$ylab)) args$ylab <- as.expression(bquote("Local" ~ L[1] ~ .(attr(x, "mode"))~"(uniform margin)"))
      do.call(plot, c(list(NA), args))
      do.call(graphics::matlines, c(list(attr(x,"alpha"), t(x_unif)), args))
    }else{
      x_local_change <- apply(x_unif, 1, function(vec) max(vec) - min(vec))
      if(is.null(args$xlim)) args$xlim <- range(attr(x, "alpha")) + c(0, 0.1*diff(range(attr(x,"alpha"))))
      if(is.null(args$ylim)) args$ylim <- c(0, 1)
      if(is.null(args$xlab)) args$xlab <- "Locality level"
      if(is.null(args$ylab)) args$ylab <- as.expression(bquote("Local" ~ L[1] ~ .(attr(x, "mode"))~"(uniform margin)"))
      do.call(plot, c(list(NA), args))
      if(sum(x_local_change < threshold) > 0) do.call(graphics::matlines, c(list(attr(x,"alpha"), t(x_unif[x_local_change < threshold,,drop=FALSE]), col = "gray", lty = 1)))
      if(sum(x_local_change >= threshold) > 0){
        do.call(graphics::matlines, c(list(attr(x,"alpha"), t(x_unif[x_local_change >= threshold,,drop=FALSE]),
                                 col = grDevices::palette()[(1:sum(x_local_change >= threshold) - 1) %% 8 + 1],
                                 lty = 1, lwd = 2), args))
        lab.threshold <- NULL
        if(is.null(attr(x,"label.igraph"))){
          lab.threshold <- attr(x,"label")[which(x_local_change >= threshold)]
        }else{
          lab.threshold <- attr(x,"label.igraph")[which(x_local_change >= threshold)]
        }
        text(max(attr(x,"alpha")), x_unif[which(x_local_change >= threshold), ncol(x_unif)],
             labels = lab.threshold,
             pos = 4, col = grDevices::palette()[(1:sum(x_local_change >= threshold) - 1) %% 8 + 1])
      }
    }
  }
}

#' @name L1cent
#' @aliases L1prestige L1pres
#' @title L1 Centrality/Prestige
#'
#' @description
#' Computes \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality or
#' \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige for each
#' vertex. The \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
#' centrality/prestige is a graph centrality/prestige measure defined for the
#' vertices of a graph. It is (roughly) defined by (1 \eqn{-} minimum
#' multiplicity required for a selected vertex to become the median of the
#' graph). For directed graphs,
#' \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality quantifies
#' the prominence of a vertex in *making* a choice and
#' \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige quantifies
#' the prominence of a vertex in *receiving* a choice. For undirected graphs,
#' the two measures are identical.
#'
#' @note
#' The function is valid only for connected graphs. If the graph is directed, it
#' must be strongly connected.
#'
#' @details
#' Suppose that \code{g} is a (strongly) connected graph consisting of
#' \ifelse{html}{\out{<i>n</i>}}{\eqn{n}} vertices
#' \ifelse{html}{\out{<i>v</i><sub>1</sub>, ...,
#' <i>v<sub>n</sub></i>}}{{\eqn{v_1,\dots,v_n}}}
#' whose multiplicities (vertex weights) are \eqn{\eta_1,\dots,\eta_n \geq 0}, respectively,
#' and \eqn{\eta_{\cdot} = \sum_{k=1}^n \eta_k > 0}.
#'
#' The centrality median vertex of this graph is the node minimizing the
#' weighted sum of distances. That is,
#' \ifelse{html}{\out{<i>v<sub>i</sub></i>}}{{\eqn{v_i}}} is the centrality
#' median vertex if
#' \deqn{
#'  \sum_{k=1}^{n} \eta_k d(v_i, v_k)
#' }
#' is minimized, where \eqn{d(v_i,v_k)} denotes the geodesic (shortest path)
#' distance from \eqn{v_i} to \eqn{v_k}. See [igraph::distances()] for
#' algorithms for computing geodesic distances between vertices. When the
#' indices are swapped to \eqn{d(v_k, v_i)} in the display above, we call the
#' node minimizing the weighted sum as the prestige median vertex. When the
#' graph is undirected, the prestige median vertex and the centrality median
#' vertex coincide, and we call it the graph median, following Hakimi (1964).
#'
#' The \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality for an
#' arbitrary node \ifelse{html}{\out{<i>v<sub>i</sub></i>}}{{\eqn{v_i}}} is
#' defined as \sQuote{one minus the minimum weight that is required to make it a
#' centrality median vertex.} This concept of centrality is closely related to the
#' data depth for ranking multivariate data, as defined in Vardi and Zhang
#' (2000). It turns out that the following formula computes the
#' \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality for the
#' vertex \ifelse{html}{\out{<i>v<sub>i</sub></i>}}{{\eqn{v_i}}}:
#' \deqn{
#'  1-\mathcal{S}(\texttt{g})\max_{j\neq i}\left\{\frac{\sum_{k=1}^{n}\eta_k (d(v_i,v_k) - d(v_j,v_k)) }{\eta_{\cdot}d(v_j,v_i)}\right\}^{+},
#' }
#' where \eqn{\{\cdot\}^{+}=\max(\cdot,0)} and \eqn{\mathcal{S}(\texttt{g}) =
#' \min_{i\neq j} d(v_i,v_j)/d(v_j,v_i)}. The
#' \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality of a vertex
#' is in \eqn{[0,1]} by the triangle inequality, and the centrality median
#' vertex has centrality 1. The
#' \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige is defined
#' analogously, with the indices inside the distance function swapped.
#'
#' For an undirected graph, \eqn{\mathcal{S}(\texttt{g}) = 1} since the distance
#' function is symmetric. Moreover,
#' \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality and
#' \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige measures concide.
#'
#' For details, refer to Kang and Oh (2026a) for undirected graphs, and Kang and
#' Oh (2026b) for directed graphs.
#'
#' @param g An \code{igraph} graph object or a distance matrix.
#'  * When \code{g} is an \code{igraph} object, the graph must be connected.
#'  For a directed graph, it must be strongly connected.
#'  * When \code{g} is a matrix, it should represent a distance matrix.
#'  All entries of the distance matrix must be finite. Here, the
#'   \ifelse{html}{\out{(<i>i,j</i>)}}{\eqn{(i,j)}} component of the distance
#'   matrix is the geodesic distance from the
#'   \ifelse{html}{\out{<i>i</i>}}{\eqn{i}}th vertex to the
#'   \ifelse{html}{\out{<i>j</i>}}{\eqn{j}}th vertex.
#' @param vertex_weight An optional nonnegative multiplicity (weight) vector for (vertex)
#'   weighted networks. The sum of its components must be positive. If set to
#'   \code{NULL} (the default), all vertices will have the same positive weight
#'   (multiplicity) of 1, i.e., \code{g} is treated as a vertex unweighted graph. The
#'   length of the \code{vertex_weight} must be equivalent to the number of vertices.
#' @param mode A character string. For an undirected graph, either choice gives
#'   the same result.
#'  * `centrality` (the default): \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
#'  centrality (prominence of each vertex in terms of \emph{making} a choice) is
#'  used for analysis.
#'  * `prestige`: \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
#'  prestige (prominence of each vertex in terms of \emph{receiving} a choice)
#'  is used for analysis.
#' @param edge_weight_transform An optional function to transform the edge weights
#'   when `g` is an `igraph` object and an edge weight attribute exists. This
#'   argument is ignored when `g` is a distance matrix.
#' @return \code{L1cent()} returns an object of class \code{L1cent}. It is a
#'   numeric vector whose length is equivalent to the number of vertices in the
#'   graph \code{g}. Each component of the vector is the
#'   \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality (if
#'   \code{mode = "centrality"}) or the
#'   \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige (if
#'   \code{mode = "prestige"}) of each vertex in the given graph.
#'
#'  \code{print.L1cent()} prints
#'  \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality or
#'  \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige values and
#'  returns them invisibly.
#'
#' @export
#' @seealso [L1centLOC()], [L1centNB()], [L1centMDS()], [L1centEDGE()],
#'   [L1centGROUP()] for
#'   \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality- or
#'   prestige-based analysis. See [L1centrality-package] for each function's
#'   support range.
#'
#'   [igraph::betweenness()], [igraph::closeness()],
#'   [igraph::degree()], [igraph::eigen_centrality()] for centrality measures.
#'
#'   [Heterogeneity] for drawing the Lorenz curve and computing the Gini
#'   coefficient.
#'
#' @examples
#' # igraph object and distance matrix as an input lead to the same result
#' vertex_weight <- igraph::V(MCUmovie)$worldwidegross
#' cent_igraph <- L1cent(MCUmovie, vertex_weight = vertex_weight)
#' cent_matrix <- L1cent(igraph::distances(MCUmovie), vertex_weight = vertex_weight)
#' all(cent_igraph == cent_matrix)
#'
#' # Top 6 vertices with the highest L1 centrality
#' utils::head(sort(cent_igraph, decreasing = TRUE))
#'
#' # Lorenz curve
#' plot(cent_igraph)
#'
#' # Summary statistics
#' summary(cent_igraph)
#'
#' @references S. L. Hakimi. Optimum locations of switching centers and the
#'   absolute centers and medians of a graph. \emph{Operations Research},
#'   12(3): 450--459, 1964.
#'
#'   S. Kang and H.-S. Oh. On a notion of graph centrality based on
#'   \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} data depth.
#'   \emph{Journal of the American Statistical Association}, 121(553): 400--412, 2026a.
#'
#'   S. Kang and H.-S. Oh.
#'   \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prominence measures
#'   for directed graphs. \emph{The American Statistician}, 80(2): 301--309, 2026b.
#'
#'   Y. Vardi and C.-H. Zhang. The multivariate
#'   \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}-median and
#'   associated data depth. \emph{Proceedings of the National Academy of Sciences},
#'   97(4): 1423--1426, 2000.
L1cent <- function(g, vertex_weight, mode, edge_weight_transform) UseMethod("L1cent")

#' @name L1cent
#' @exportS3Method L1cent igraph
L1cent.igraph <- function(g, vertex_weight = NULL, mode = c("centrality", "prestige"), edge_weight_transform = NULL) {
  validate_igraph(g, checkdir = FALSE)
  mode <- match.arg(tolower(mode), choices = c("centrality", "prestige"))

  new_weight <- edge_weight_transform0(g, edge_weight_transform)
  if(!is.null(new_weight)) igraph::E(g)$weight <- new_weight
  D <- igraph::distances(g, mode = "out")
  L1cent.matrix(g = D, vertex_weight = vertex_weight, mode = mode)
}

#' @name L1cent
#' @exportS3Method L1cent matrix
L1cent.matrix <- function(g, vertex_weight = NULL, mode = c("centrality", "prestige"), edge_weight_transform = NULL) {
  eta <- vertex_weight
  calls <- sys.calls()
  fnames <- sapply(calls, function(cl) as.character(cl[[1]]))
  # print(fnames)
  if(all(fnames[1:2] == c("L1cent", "L1cent.matrix")))
    message("DISTANCE matrix received")

  if(is.null(eta)) eta <- rep(1,ncol(g))
  validate_matrix(g, eta, checkdir = FALSE)
  mode <- match.arg(tolower(mode), choices = c("centrality", "prestige"))

  n <- ncol(g)
  sg <- ifelse(isSymmetric.matrix(g), 1,
               min((g/t(g))[upper.tri(g) | lower.tri(g)]))

  if(identical(mode, "centrality")){
    geta1 <- matrix(rep(colSums(t(g) * eta), n), ncol = n)
    res <- rep(1, n) - sg / sum(eta) *
      apply((geta1 - t(geta1)) / (t(g) + diag(rep(Inf, n))), 1,
            function(r) max(c(r, 0)))
    l1 <- pmin(pmax(res,0),1)
  }else{ # prestige
    geta2 <- matrix(rep(colSums(g * eta), n), ncol = n)
    res <- rep(1, n) - sg / sum(eta) *
      apply((geta2 - t(geta2)) / (g + diag(rep(Inf, n))), 1,
            function(r) max(c(r, 0)))
    l1 <- pmin(pmax(res,0),1)
  }
  structure(l1, class = "L1cent", mode = mode)
}

#' @name L1cent
#' @aliases print.L1cent
#'
#' @param x An \code{L1cent} object, obtained as a result of the function
#'   \code{L1cent()}.
#' @param ... Further arguments passed to or from other methods.
#' @export
print.L1cent <- function(x, ...){
  cat("L1 ", attr(x, "mode"), ":\n", sep = "")
  print.default(c(x), ...)
  return(invisible(x))
}

#' @name L1cent
#' @aliases plot.L1cent
#'
#' @param y An optional argument providing the coordinates for a scatter plot.
#'   It could be an object of class \code{L1cent} or \code{L1centLOC}, or a
#'   numeric vector.
#' @param add A logical value. This argument is considered only when drawing a Lorenz curve.
#'  * \code{TRUE}: add the Lorenz curve to an already existing plot.
#'  * \code{FALSE} (the default): draw the Lorenz curve to a new graphic device.
#' @return `plot.L1cent()` draws a Lorenz curve (the group heterogeneity plot)
#'   and returns an invisible copy of a Gini coefficient (the group
#'   heterogeneity index) if argument \code{y} is not supplied. Otherwise, it
#'   draws a scatter plot.
#' @export
plot.L1cent <- function(x, y = NULL, add = FALSE, ...){
  args <- list(...)
  if(is.null(y)){
    args$add <- add
    if(is.null(args$main)) args$main <- "Lorenz plot"
    do.call(Lorenz_plot, c(list(x), args))
  }else{
    if(methods::is(y,"L1centLOC")){
      if(is.null(args$xlab)) args$xlab <- as.expression(bquote(L[1] ~ .(attr(x, "mode"))))
      if(is.null(args$ylab)) args$ylab <- as.expression(bquote("Local" ~ L[1] ~ .(attr(y, "mode")) ~ "(alpha =" ~ .(attr(y, "alpha"))*")"))
      do.call(plot, c(list(as.numeric(x), as.numeric(unlist(y))), args))
    }else if(methods::is(y,"L1cent")){
      if(is.null(args$xlab)) args$xlab <- as.expression(bquote(L[1] ~ .(attr(x, "mode"))))
      if(is.null(args$ylab)) args$ylab <- as.expression(bquote(L[1] ~ .(attr(y, "mode"))))
      do.call(plot, c(list(as.numeric(x), as.numeric(y)), args))
    }else{
      if(is.null(args$xlab)) args$xlab <- as.expression(bquote(L[1] ~ .(attr(x, "mode"))))
      if(is.null(args$ylab)) args$ylab <- "y"
      do.call(plot, c(list(as.numeric(x), y), args))
    }
  }
}

# logical function for checking whether all elements of a vector are integral,
# from the documentation of `is.integer()`
# NOT exported
is.wholenumber <- function(x, tol = .Machine$double.eps^0.5)  abs(x - round(x)) < tol

#' @name group_reduce
#' @title Group Reduced Graph
#'
#' @description
#' Computes the vertex multiplicities (vertex weights) and the distance matrix of the
#' group reduced graph. The group reduced graph is constructed by replacing a
#' group of vertices in the original graph by a single \sQuote{pseudo-vertex}.
#'
#' @note
#' Multiple edges (edges with the same head and tail vertices) are not allowed,
#' because they make the edge weight setting procedure confusing.
#'
#' @details
#' The group reduced graph is constructed by replacing the vertices indicated in
#' the argument \code{nodes} with a single \sQuote{pseudo-vertex}. The
#' multiplicity (vertex weight) of this new vertex is set to the sum of the
#' multiplicities of the vertices within \code{nodes}. An edge from the
#' pseudo-vertex to any vertex that is not in \code{nodes}, say
#' \ifelse{html}{\out{<i>v</i>}}{\eqn{v}}, is created in the group reduced graph if
#' there is at least one edge from the vertices in \code{nodes} to
#' \ifelse{html}{\out{<i>v</i>}}{\eqn{v}} in the original graph. The weight of
#' this newly added edge is determined using one of the following methods:
#' \itemize{
#' \item Minimum method: The edge weight from the pseudo-vertex to
#' \ifelse{html}{\out{<i>v</i>}}{\eqn{v}} is set to the minimum of the edge
#' weights of the edges between the vertices in \code{nodes} to
#' \ifelse{html}{\out{<i>v</i>}}{\eqn{v}} in the original graph.
#' \item Maximum method: The edge weight from the pseudo-vertex to
#' \ifelse{html}{\out{<i>v</i>}}{\eqn{v}} is set to the maximum of the edge
#' weights of the edges between the vertices in \code{nodes} to
#' \ifelse{html}{\out{<i>v</i>}}{\eqn{v}} in the original graph.
#' \item Average method: The edge weight from the pseudo-vertex to
#' \ifelse{html}{\out{<i>v</i>}}{\eqn{v}} is set to the average of the edge
#' weights of the edges between the vertices in \code{nodes} to
#' \ifelse{html}{\out{<i>v</i>}}{\eqn{v}} in the original graph.
#' }
#' An edge from \ifelse{html}{\out{<i>v</i>}}{\eqn{v}} to the pseudo-vertex is
#' set in a similar manner. For details, refer to Kang (2025).
#'
#' @inheritParams L1cent
#' @param g An \code{igraph} graph object or a distance matrix. When \code{g} is
#'  a distance matrix, the
#'   \ifelse{html}{\out{(<i>i,j</i>)}}{\eqn{(i,j)}} component of the distance
#'   matrix is the geodesic distance from the
#'   \ifelse{html}{\out{<i>i</i>}}{\eqn{i}}th vertex to the
#'   \ifelse{html}{\out{<i>j</i>}}{\eqn{j}}th vertex.
#' @param nodes A vector of integers. Each integer indicates the index of the
#'   vertex.
#' @param method A character string. It specifies the method of setting the edge
#'   weight between the pseudo-vertex and the other vertices. Note that the S3
#'   method for the \code{matrix} class only supports the `minimum` option. This
#'   is because it is not possible to derive the group reduced graph's distance
#'   matrix from the original distance matrix when using the maximum or average
#'   method. On the other hand, the group reduced graph's distance matrix can be
#'   derived from the original distance matrix when the minimum method is used.
#'   See the discussion in Kang (2025).
#'  * `minimum` (the default): the minimum method is used in setting the edge weights.
#'  * `maximum`: the maximum method is used in setting the edge weights.
#'  * `average`: the average method is used in setting the edge weights.
#' @return A list consisting of three objects:
#'  \itemize{
#'  \item \sQuote{distmat}: A matrix representing the group reduced graph's
#'  distance matrix, where the first row and column correspond to the
#'  pseudo-vertex.
#'  \item \sQuote{vertex_weight}: A vector of the group reduced graph's vertex
#'  multiplicity. The first element corresponds to the pseudo-vertex.
#'  \item \sQuote{label}: A vector of the vertex names specified by \code{nodes}.
#' }
#'
#' @export
#' @seealso [L1cent()] for
#'   \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality/prestige.
#'   [L1centGROUP()] internally uses \code{group_reduce()}.
#' @examples
#' # Group reduced graph of the 'Iron Man' series using the minimum method
#' vertex_weight <- igraph::V(MCUmovie)$worldwidegross
#' ironman_series <- c(1,3,7)
#' reduced_graph <- group_reduce(MCUmovie, nodes = ironman_series, vertex_weight = vertex_weight)
#' reduced_graph$distmat[1:3,1:3]
#' reduced_graph$label
#'
#' # Multiplicity of the pseudo-vertex equals the sums of the replaced vertices' multiplicities
#' reduced_graph$vertex_weight[1] == sum(vertex_weight[ironman_series])
#' @references S. Kang. \emph{Topics in Non-Euclidean Dimension Reduction}. PhD thesis,
#'   Seoul National University, 2025.
group_reduce <- function(g, nodes, vertex_weight, method, edge_weight_transform) UseMethod("group_reduce")

#' @name group_reduce
#' @exportS3Method group_reduce igraph
group_reduce.igraph <- function(g, nodes, vertex_weight = NULL, method = c("minimum", "maximum", "average"), edge_weight_transform = NULL){
  eta <- vertex_weight
  if(is.null(eta)) eta <- rep(1, igraph::vcount(g))
  # validate_igraph(g, checkdir = FALSE)
  if(igraph::any_multiple(g))
    stop("Multiple edges are not allowed")
  if(length(eta) != igraph::vcount(g))
    stop("Length of eta differs from the number of vertices")
  if(any(eta < 0))
    stop("Entries of eta must be non-negative")
  if(sum(eta) <= 0)
    stop("sum(eta) must be positive")

  n <- igraph::vcount(g)
  if(!all(is.wholenumber(nodes)) | !all(nodes >= 1 & nodes <= n))
    stop(paste0("Invalid 'nodes' argument: Elements should be integers between 1 and ",n))
  nodes <- sort(unique(nodes))
  method <- match.arg(tolower(method), choices = c("minimum", "maximum", "average"))

  new_weight <- edge_weight_transform0(g, edge_weight_transform)
  if(!is.null(new_weight)) igraph::E(g)$weight <- new_weight
  if(is.null(igraph::E(g)$weight)) igraph::E(g)$weight <- rep(1, igraph::ecount(g))
  A <- igraph::as_adjacency_matrix(g, attr = "weight")
  if(is.null(colnames(A))) colnames(A) <- rownames(A) <- 1:n
  if(length(nodes) == n){
    Anew <- matrix(0)
  }else if(identical(method, "minimum")){
    A[A == 0] <- Inf
    Anew <- cbind(c(0,apply(A[-nodes, nodes, drop = FALSE], 1, min)),
                  rbind(apply(A[nodes, -nodes, drop = FALSE], 2, min), A[-nodes, -nodes, drop = FALSE]))
    Anew[Anew == Inf] <- 0.0
  }else if(identical(method, "maximum")){
    Anew <- cbind(c(0,apply(A[-nodes, nodes, drop = FALSE], 1, max)),
                  rbind(apply(A[nodes, -nodes, drop = FALSE], 2, max), A[-nodes, -nodes, drop = FALSE]))
  }else{ # average
    Anew <- cbind(c(0,apply(A[-nodes, nodes, drop = FALSE], 1, function(vec) ifelse(all(vec == 0), 0, mean(vec[vec != 0])))),
                  rbind(apply(A[nodes, -nodes, drop = FALSE], 2, function(vec) ifelse(all(vec == 0), 0, mean(vec[vec != 0]))), A[-nodes, -nodes, drop = FALSE]))
  }
  gnew <- igraph::graph_from_adjacency_matrix(Anew, weighted = TRUE)
  igraph::V(gnew)$name[1] <- "Pseudo-vertex"
  D <- igraph::distances(gnew, mode = "out")
  return(list(distmat = D, vertex_weight = c(sum(eta[nodes]), eta[-nodes]),
              label=if(is.null(igraph::V(g)$name)) nodes else igraph::V(g)$name[nodes]))
}

#' @name group_reduce
#' @exportS3Method group_reduce matrix
group_reduce.matrix <- function(g, nodes, vertex_weight = NULL, method = "minimum", edge_weight_transform = NULL){
  eta <- vertex_weight
  if(is.null(eta)) eta <- rep(1,ncol(g))
  # validate_matrix(g, eta, checkdir = FALSE)
  if(length(eta) != ncol(g))
    stop("Length of eta differs from the number of vertices")
  if(any(eta < 0))
    stop("Entries of eta must be non-negative")
  if(sum(eta) <= 0)
    stop("sum(eta) must be positive")

  calls <- sys.calls()
  fnames <- sapply(calls, function(cl) as.character(cl[[1]]))
  # print(fnames)
  if(all(fnames[1:2] == c("group_reduce", "group_reduce.matrix")))
    message("DISTANCE matrix received")

  n <- ncol(g)
  if(!all(is.wholenumber(nodes)) | !all(nodes >= 1 & nodes <= n))
    stop(paste0("Invalid nodes argument: Elements should be integers between 1 and ",n))
  nodes <- sort(unique(nodes))
  method <- match.arg(tolower(method), choices = "minimum")

  D <- cbind(c(0,apply(g[-nodes, nodes, drop = FALSE], 1, min)),
             rbind(apply(g[nodes, -nodes, drop = FALSE], 2, min), g[-nodes, -nodes]))
  colnames(D) <- rownames(D) <- c("Pseudo-vertex", if(is.null(colnames(g))) (1:n)[-nodes] else colnames(g)[-nodes])
  D[-1,-1] <- pmin(outer(D[-1,1],D[1,-1],"+"), D[-1,-1])
  return(list(distmat = D, vertex_weight = c(sum(eta[nodes]), eta[-nodes]),
              label=if(is.null(colnames(g))) nodes else colnames(g)[nodes]))
}


################################################################################

#' @name L1centGROUP
#' @aliases L1presGROUP
#' @title Group L1 Centrality/Prestige
#'
#' @description
#' Computes group \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
#' centrality or group \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
#' prestige for the specified groups of vertices. For undirected graphs, the two
#' measures are identical.
#'
#' @note
#' The function is valid only for connected graphs. If the graph is directed, it
#' must be strongly connected. Multiple edges (edges with the same head and tail
#' vertices) are not allowed, because they make the edge weight setting procedure
#' confusing.
#'
#' @details
#' Given a group of vertices on a graph, we first construct a group reduced
#' graph by replacing the group of vertices by a single \sQuote{pseudo-vertex}
#' (see [group_reduce()] for the method of setting vertex multiplicities and
#' edge weights in the group reduced graph). The group
#' \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality (prestige)
#' of this group is defined as the
#' \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality (prestige)
#' of the pseudo-vertex in the group reduced graph.
#'
#' @inheritParams group_reduce
#' @param g An \code{igraph} graph object or a distance matrix.
#'  * When \code{g} is an \code{igraph} object, the graph must be connected.
#'  For a directed graph, it must be strongly connected.
#'  * When \code{g} is a matrix, it should represent a distance matrix.
#'  All entries of the distance matrix must be finite. Here, the
#'   \ifelse{html}{\out{(<i>i,j</i>)}}{\eqn{(i,j)}} component of the distance
#'   matrix is the geodesic distance from the
#'   \ifelse{html}{\out{<i>i</i>}}{\eqn{i}}th vertex to the
#'   \ifelse{html}{\out{<i>j</i>}}{\eqn{j}}th vertex.
#' @param nodes A vector of integers or a list of integer vectors. Each integer
#'   indicates the index of the vertex. When the argument is provided as a list,
#'   the function computes the group
#'   \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prominence for each
#'   set of vertices specified by the individual list elements. When it is given
#'   as an integer vector, the function computes the group
#'   \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prominence for the
#'   single group defined by that vector.
#' @param mode A character string. For an undirected graph, either choice gives
#'   the same result.
#'  * `centrality` (the default): \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
#'  centrality (prominence of each vertex in terms of \emph{making} a choice) is
#'  used for analysis.
#'  * `prestige`: \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
#'  prestige (prominence of each vertex in terms of \emph{receiving} a choice)
#'  is used for analysis.
#' @return `L1centGROUP()` returns an object of class `L1centGROUP`. It is a
#'   numeric value of the group
#'  \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality (if
#'  \code{mode = "centrality"}) or the group
#'  \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige (if
#'  \code{mode = "prestige"}) of the specified group of vertices.
#'
#'  `print.L1centGROUP()` prints group
#'  \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality or group
#'  \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige value and
#'  returns it invisibly.
#'
#' @export
#' @seealso [L1cent()] for \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
#'   centrality/prestige, [group_reduce()] for details on the minimum, maximum, and average methods.
#' @examples
#' # Group L1 centrality of the 'Spider-Man' series
#' vertex_weight <- igraph::V(MCUmovie)$worldwidegross
#' L1centGROUP(MCUmovie, nodes = list(16, c(16,23,27)), vertex_weight = vertex_weight)
#' @references S. Kang. \emph{Topics in Non-Euclidean Dimension Reduction}. PhD thesis,
#'   Seoul National University, 2025.
L1centGROUP <- function(g, nodes, vertex_weight, mode, method, edge_weight_transform) UseMethod("L1centGROUP")

#' @name L1centGROUP
#' @exportS3Method L1centGROUP igraph
L1centGROUP.igraph <- function(g, nodes, vertex_weight = NULL, mode = c("centrality", "prestige"),
                               method = c("minimum", "maximum", "average"), edge_weight_transform = NULL) {
  eta <- vertex_weight
  mode <- match.arg(tolower(mode), choices = c("centrality", "prestige"))
  method <- match.arg(tolower(method), choices = c("minimum", "maximum", "average"))
  validate_igraph(g, checkdir = FALSE)
  if(!is.list(nodes)) nodes <- list(nodes)
  res.value <- numeric(length(nodes))
  label.list <- vector("list", length = length(nodes))
  eta0 <- eta
  for(nn in seq_along(nodes)){
    g_reduce <- group_reduce.igraph(g = g, nodes = nodes[[nn]], vertex_weight = eta0, method = method, edge_weight_transform = edge_weight_transform)
    D <- g_reduce$distmat
    eta <- g_reduce$vertex_weight
    label.list[[nn]] <- g_reduce$label
    n <- ncol(D)
    sg <- ifelse(isSymmetric.matrix(D), 1,
                 min((D/t(D))[upper.tri(D) | lower.tri(D)]))
    if(identical(mode, "centrality")){
      geta1 <- colSums(t(D) * eta)
      res <- 1 - sg / sum(eta) *
        max(c((geta1[1] - geta1[-1]) / D[-1,1]), 0)
      res.value[nn] <- min(max(res,0),1)
    }else{ # prestige
      geta2 <- colSums(D * eta)
      res <- 1 - sg / sum(eta) *
        max(c((geta2[1] - geta2[-1]) / D[1,-1]), 0)
      res.value[nn] <- min(max(res,0),1)
    }
    # nodes <- sort(unique(nodes))
  }
  return(structure(res.value,
                   class = "L1centGROUP",
                   mode = mode,
                   label = label.list,
                   method = method))
}

#' @name L1centGROUP
#' @exportS3Method L1centGROUP matrix
L1centGROUP.matrix <- function(g, nodes, vertex_weight = NULL, mode = c("centrality", "prestige"),
                               method = "minimum", edge_weight_transform = NULL) {
  eta <- vertex_weight
  mode <- match.arg(tolower(mode), choices = c("centrality", "prestige"))
  method <- match.arg(tolower(method), choices = c("minimum"))
  if(is.null(eta)) eta <- rep(1, ncol(g))
  validate_matrix(g, eta, checkdir = FALSE)

  calls <- sys.calls()
  fnames <- sapply(calls, function(cl) as.character(cl[[1]]))
  # print(fnames)
  if(all(fnames[1:2] == c("L1centGROUP", "L1centGROUP.matrix")))
    message("DISTANCE matrix received")

  if(!is.list(nodes)) nodes <- list(nodes)
  res.value <- numeric(length(nodes))
  label.list <- vector("list", length = length(nodes))
  eta0 <- eta
  for(nn in seq_along(nodes)){
    g_reduce <- group_reduce.matrix(g = g, nodes = nodes[[nn]], vertex_weight = eta0, method = method)
    D <- g_reduce$distmat
    eta <- g_reduce$vertex_weight
    label.list[[nn]] <- g_reduce$label
    n <- ncol(D)
    sg <- ifelse(isSymmetric.matrix(D), 1,
                 min((D/t(D))[upper.tri(D) | lower.tri(D)]))

    if(identical(mode, "centrality")){
      geta1 <- colSums(t(D) * eta)
      res <- 1 - sg / sum(eta) *
        max(c((geta1[1] - geta1[-1]) / D[-1,1]), 0)
      res.value[nn] <- min(max(res,0),1)
    }else{ # prestige
      geta2 <- colSums(D * eta)
      res <- 1 - sg / sum(eta) *
        max(c((geta2[1] - geta2[-1]) / D[1,-1]), 0)
      res.value[nn] <- min(max(res,0),1)
    }
    # nodes <- sort(unique(nodes))
  }
  return(structure(res.value,
                   class = "L1centGROUP",
                   mode = mode,
                   label = label.list,
                   method = method))
}

#' @name L1centGROUP
#' @aliases print.L1centGROUP
#'
#' @param x An \code{L1centGROUP} object, obtained as a result of the function
#'   \code{L1centGROUP()}.
#' @param ... Further arguments passed to or from other methods. This argument
#'   is ignored here.
#' @export
print.L1centGROUP <- function(x, ...){
  # labs <- rep(" ", length(attr(x, "label")) * 2 - 1)
  # labs[(1:length(attr(x, "label"))) * 2 - 1] <- sQuote(attr(x, "label"))
  # labs[1] <- paste0("(", labs[1])
  # labs[length(labs)] <- paste0(labs[length(labs)], ")")
  labs <- attr(x, "label")
  labs <- paste0(
    ifelse(length(attr(x, "label")) == 1,"","("),
    paste(
      sapply(labs, function(x) {
        paste0("(", paste0('"', x, '"', collapse = " "), ")")
      }),
      collapse = ", "
    ),
    ifelse(length(attr(x, "label")) == 1,"",")")
  )

  mes <- paste0("group L1 ", attr(x, "mode"), " of ", length(attr(x, "label")),
      ifelse(length(attr(x, "label")) == 1, " group ", " groups "),
      labs, " with ", sQuote(attr(x, "method")), " method:")
  cat(strwrap(mes, width = getOption("width")), sep = "\n")
  print.default(c(x), ...)
  return(invisible(x))
}

#' @name L1centMDS
#' @title Fitting a Target Plot
#'
#' @description
#' \code{L1centMDS()} and \code{plot.L1centMDS()} are used together to draw a
#' target plot, which is a target-shaped 2D plot that aids in the visual
#' inspection of an undirected graph using the
#' \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality. See Kang
#' and Oh (2026) for a formal definition of a target plot.
#'
#' @details
#' Denoting the \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
#' centrality of vertex \eqn{i} as \eqn{c_i\in(0,1]}, a point representing that vertex is placed
#' on a concentric circle with radius \eqn{r_i =
#' -\log(c_i)}. Representing each vertex as \eqn{(r_i, \theta_i)} (in circular
#' coordinates), the values of \eqn{\theta_i} are derived using nonmetric
#' multidimensional scaling proposed in Kruskal (1964a,b). The initial
#' configuration is derived using classical multidimensional scaling
#' ([stats::cmdscale()]). A gradient descent algorithm is employed in deriving
#' optimal \eqn{\theta_i}s.
#'
#' @note
#' The function \code{L1centMDS()} is valid only for undirected and connected
#' graphs. Also, \code{L1centMDS()} only considers graphs with equal vertex
#' multiplicities.
#'
#' @param g An \code{igraph} graph object or a distance matrix.
#' * When \code{g} is an \code{igraph} object, the graph must be undirected and connected.
#' * When \code{g} is a matrix, it should be symmetric and represent a distance matrix.
#' All entries of the distance matrix must be finite.
#' @param edge_weight_transform An optional function to transform the edge weights
#'   when `g` is an `igraph` object and an edge weight attribute exists. This
#'   argument is ignored when `g` is a distance matrix.
#' @param tol A numerical tolerance. The gradient descent method terminates if
#'   the relative magnitude of the gradient falls below \code{tol} as in Kruskal
#'   (1964b). By default set to \ifelse{html}{\out{10<sup>-5</sup>}}{\eqn{10^{-5}}}.
#' @param maxiter A number of maximum iteration allowances for the gradient descent
#'   algorithm. By default set to 1000.
#' @param x An \code{L1centMDS} object, obtained as a result of the function
#'   \code{L1centMDS()}.
#' @param verbose A boolean.
#'  * `TRUE` (the default): for each iteration, prints (1) current number of
#'  iterations, (2) current stress, and (3) relative magnitude of the gradient
#'  to the console. At the end, the final message is printed to the console; total
#'  number of iterations and final stress.
#'  * `FALSE`: suppress message to the console.
#' @param zoom A numerical value on how much to zoom-in the plot. By default set
#'   to 1 (no zoom).
#' @param main Title of the plot. If set to \code{NULL} (the default), the title
#'   prints \dQuote{Target plot / Stress = X}.
#' @param show_edge A boolean.
#'  * `TRUE`: edges are shown in the target plot. When `x`, the `L1centMDS`
#'  object, is generated from a matrix object `g`, this argument is ignored
#'  since `g` does not carry information about the edge connections. That is, edges
#'  can only be drawn if the `L1centMDS` object is generated from an `igraph` object.
#'  * `FALSE` (the default): edges are not shown in the target plot.
#' @param ... Further arguments passed to or from other methods.
#'   * `plot()` method: Further graphical parameters supplied to the internal
#'   [base::plot()] (for points), [graphics::text()] (for labels), and
#'   [graphics::segments()] (for edges) functions. See [graphics::par()]
#'   document. To supply an argument to the first one, use the prefix
#'   \sQuote{\code{plot.}}, for the second one, \sQuote{\code{text.}}, and for
#'   the last one, \sQuote{\code{edge.}}. For instance, \code{plot.cex = 1} sets
#'   the size of the point, whereas \code{text.cex = 1} sets the size of the
#'   label.
#'   * `print()` method: This argument is ignored.
#'
#' @return \code{L1centMDS()} returns an object of class \code{L1centMDS}. It is a list
#'   consisting of three vectors:
#' \itemize{
#'  \item \sQuote{radius}: Radius of a point representing each vertex in the
#'  target plot's circular coordinate system, i.e., \eqn{-\log(L_1\text{
#'  centrality})} for each vertex.
#'  \item \sQuote{theta}: Angle (in radians) of a point representing each vertex
#'  in the target plot's circular coordinate system.
#'  \item \sQuote{stress}: Stress measure defined in Kruskal (1964a).
#' }
#'
#' \code{plot.L1centMDS()} draws a target plot. Four concentric circles denote
#' the 1st to 4th quartiles of the radius, and the values of the
#' \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality quartiles
#' are shown in red text. Note that red texts denote the
#' \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality quartiles,
#' \emph{not} radius quartiles.
#'
#' `print.L1centMDS()` prints number of iterations it took to fit a target plot.
#'
#' @export
#' @seealso [L1cent()] for
#'   \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality/prestige,
#'   [MASS::isoMDS()] and [stats::cmdscale()] for multidimensional scaling
#'   methods.
#' @examples
#' parameters <- L1centMDS(MCUmovie, verbose = FALSE)
#' plot(parameters)
#' @references S. Kang and H.-S. Oh. On a notion of graph centrality based on
#'   \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} data depth.
#'   \emph{Journal of the American Statistical Association}, 121(553): 400--412, 2026.
#'
#'   J. B. Kruskal. Multidimensional scaling by optimizing goodness of fit to a
#'   nonmetric hypothesis. \emph{Psychometrika}, 29(1): 1--27, 1964a.
#'
#'   J. B. Kruskal. Nonmetric multidimensional scaling: a numerical method.
#'   \emph{Psychometrika}, 29(2): 115--129, 1964b.
L1centMDS <- function(g, tol, maxiter, verbose, edge_weight_transform) UseMethod("L1centMDS")

#' @name L1centMDS
#' @exportS3Method L1centMDS igraph
L1centMDS.igraph <- function(g, tol = 1e-5, maxiter = 1000, verbose = TRUE, edge_weight_transform = NULL){
  validate_igraph(g)

  new_weight <- edge_weight_transform0(g, edge_weight_transform)
  if(!is.null(new_weight)) igraph::E(g)$weight <- new_weight
  D <- igraph::distances(g)
  attr(D, "color.igraph") <- igraph::V(g)$color
  attr(D, "label.igraph") <- igraph::V(g)$label
  res <- L1centMDS.matrix(D, tol, maxiter, verbose)
  attr(res, "edgelist") <- igraph::as_edgelist(g)
  return(res)
}

#' @name L1centMDS
#' @exportS3Method L1centMDS matrix
L1centMDS.matrix <- function(g, tol = 1e-5, maxiter = 1000, verbose = TRUE, edge_weight_transform = NULL){
  eta <- rep(1,ncol(g))
  validate_matrix(g, eta)

  calls <- sys.calls()
  fnames <- sapply(calls, function(cl) as.character(cl[[1]]))
  # print(fnames)
  if(all(fnames[1:2] == c("L1centMDS", "L1centMDS.matrix")))
    message("DISTANCE matrix received")

  stepsize <- 0.2
  n <- ncol(g)
  label <- rownames(g)
  if(is.null(label)) label <- 1:nrow(g)
  dist.original.vec <- g[upper.tri(g)]
  cent <- L1cent(g, vertex_weight = eta)
  radius <- c(-log(cent))

  # initialize using classical MDS
  init <- stats::cmdscale(g)
  temp <- t(init)-init[which.min(radius),]
  # add small errors for stability
  params <- c(atan2(temp[2,],temp[1,])) + stats::rnorm(n,0,(pi/32)^2)

  # iterate until convergence
  iter.count <- 1
  stress <- Inf
  while(TRUE){
    # current distance matrix
    DIST <- as.matrix(stats::dist(cbind(radius*cos(params),radius*sin(params))))
    DIST.vec <- DIST[upper.tri(DIST)]

    # estimate HAT.DIST
    dist.original.group <- match(dist.original.vec, sort(unique(dist.original.vec)))
    ngroup <- length(unique(dist.original.group))
    group.mean.n <- t(stats::aggregate(DIST.vec,by=list(dist.original.group),function(s)c(mean(s),length(s)))[,2])

    group.mean.n[1,] <- .dhatestimate(group.mean.n[2,],group.mean.n[1,])

    HAT.DIST.vec <- group.mean.n[1,dist.original.group]
    HAT.DIST <- matrix(0,nrow=n,ncol=n)
    HAT.DIST[upper.tri(HAT.DIST)] <- HAT.DIST.vec
    HAT.DIST <- HAT.DIST + t(HAT.DIST)

    # current stress
    Tstar <- sum(DIST.vec^2)
    Sstar <- sum((DIST.vec-HAT.DIST.vec)^2)
    new.stress <- sqrt(Sstar/Tstar)

    # gradient descent method
    R <- matrix(rep(radius,n),ncol=n)
    P <- matrix(rep(params,n),ncol=n)
    gradient <- sqrt(Tstar/Sstar)/Tstar*rowSums(
      (DIST*(1-Sstar/Tstar) - HAT.DIST)*R*t(R)*
        sin(P - t(P))/
        sqrt(R^2 + t(R^2) - 2*R*t(R)*cos(P-t(P))),na.rm=TRUE ## remove 0/0
    )
    mag.gradient <- sqrt(sum(gradient^2)/sum(radius^2))
    if(iter.count > 1){
      # Kruskal (1964b, p.121) uses complicated formula for determining the
      # stepsize. The formula is determined heuristically, based on his
      # numerical experimentation. Since we are in a different setting where
      # points are constrained to lie on a concentric circle, we use the
      # following simple formula for updating the stepsize.
      stepsize <- stepsize * 0.95
    }
    params <- (params - stepsize * gradient/mag.gradient) %% (2*pi)

    if(verbose){
      cat("\rITER ",iter.count," / Stress: ", new.stress," / ",mag.gradient,"\t\t\r",sep="")
      utils::flush.console()
    }

    if(mag.gradient <= tol){
      stress <- new.stress
      if(verbose){
        cat("\rConverged / Iteration: ",iter.count," / Stress: ",stress,"             \n",sep="")
        utils::flush.console()
      }
      break;
    }
    if(iter.count >= maxiter){
      stress <- new.stress
      if(verbose){
        cat("\rStopped after maximum iteration: ",maxiter," / Stress: ",stress,"             \n",sep="")
        utils::flush.console()
      }
      break;
    }
    stress <- new.stress
    iter.count <- iter.count + 1
  }
  return(structure(list(radius=radius,theta=params,stress=stress),
                   class="L1centMDS",
                   label=label,
                   label.igraph=attr(g, "label.igraph"),
                   color.igraph=attr(g, "color.igraph"),
                   iteration = iter.count))
}


#' @name L1centMDS
#' @aliases plot.L1centMDS
#' @export
plot.L1centMDS <- function(x, zoom=1, main=NULL, show_edge=FALSE, ...){
  radius <- x$radius
  M.radius <- max(radius)
  theta <- x$theta

  args <- list(...)
  plot.args <- list()
  text.args <- list()
  edge.args <- list()
  if(!is.null(names(args))){
    plot.args <- args[startsWith(names(args), "plot.")]
    names(plot.args) <- substr(names(plot.args),6,nchar(names(plot.args)))
    text.args <- args[startsWith(names(args), "text.")]
    names(text.args) <- substr(names(text.args),6,nchar(names(text.args)))
    edge.args <- args[startsWith(names(args), "edge.")]
    names(edge.args) <- substr(names(edge.args),6,nchar(names(edge.args)))
  }
  if(is.null(plot.args$pch)) plot.args$pch <- 20
  if(is.null(plot.args$col)) plot.args$col <- attr(x, "color.igraph")
  if(is.null(text.args$cex)) text.args$cex <- 0.5
  if(is.null(text.args$labels)){
    text.args$labels <- if(is.null(attr(x, "label.igraph"))) attr(x, "label") else attr(x, "label.igraph")
  }

  withr::local_par(list(mar = c(1,1,4,1)+0.1))
  xx <- cos(seq(0,2*pi,length.out=200))
  yy <- sin(seq(0,2*pi,length.out=200))
  grid <- stats::quantile(radius,1)
  plot((grid)*xx,(grid)*yy,col="gray",type="l",
       asp=1,xlim=c(-M.radius,M.radius)/zoom,ylim=c(-M.radius,M.radius)/zoom,
       xlab="",ylab="",axes=FALSE,
       main=ifelse(is.null(main),paste0("Target plot / Stress = ",x$stress),main))
  graphics::text(grid*cos(3*pi/4),grid*sin(3*pi/4),round(exp(-grid),4),col="red",cex=0.5)
  for(grid in stats::quantile(radius,c(0.25,0.5,0.75))){
    graphics::lines((grid)*xx,(grid)*yy,col="gray")
    graphics::text(grid*cos(3*pi/4),grid*sin(3*pi/4),round(exp(-grid),4),col="red",cex=0.5)
  }
  if(show_edge){
    el <- attr(x, "edgelist")
    lb <- attr(x, "label")
    elm <- matrix(match(el, lb), ncol = 2)
    coordm <- cbind(radius * cos(theta), radius * sin(theta))
    x0 <- coordm[elm[,1], 1]
    x1 <- coordm[elm[,2], 1]
    y0 <- coordm[elm[,1], 2]
    y1 <- coordm[elm[,2], 2]
    do.call(graphics::segments, c(list(x0, y0, x1, y1),edge.args))
    # segments(x0, y0, x1, y1)
  }
  do.call(graphics::points, c(list(radius*cos(theta),radius*sin(theta)),plot.args))
  do.call(graphics::text, c(list(radius*cos(theta),radius*sin(theta)),text.args))
}

#' @name L1centMDS
#' @aliases print.L1centMDS
#'
#' @export
print.L1centMDS <- function(x, ...){
  cat("Target plot fitted after ", attr(x, "iteration"), " iterations\n", sep = "")
  return(invisible(x))
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

.dhatestimate <- function(groupn, groupmean) {
    .Call(`_L1centrality_dhatestimate`, groupn, groupmean)
}

#' Republic of Korea's 21st National Assembly Bill Cosponsorship Network
#'
#' @description
#'
#' Network between 317 members of the Republic of Korea's 21st National
#' Assembly (May 30th, 2020--May 29th, 2024). Each member of the assembly
#' represents one vertex.
#'
#' An edge between two members is formed if there is at least one cosponsored
#' bill. The weight of this edge is given as 1/(number of cosponsored bills
#' between two members) during the first 40 months of the 21st assembly (Jun.
#' 2020--Sep. 2023).
#'
#' @name rokassembly21
#' @docType data
#' @keywords datasets
#' @usage data(rokassembly21)
#' @format An undirected, connected, and (edge) weighted \code{igraph} graph
#'   object with 317 vertices and 47,657 edges.
#'
#' Vertex attributes:
#' \itemize{
#'  \item \sQuote{name}: Pseudonyms of each member. They are in the format of
#'  the party's initial character, followed by a random number (e.g., D4). Each
#'  party's initial character is:
#'    \itemize{
#'      \item \sQuote{D}: Democratic Party of Korea.
#'      \item \sQuote{P}: People Power Party.
#'      \item \sQuote{J}: Justice Party.
#'      \item \sQuote{O}: Others (Basic Income Party, Hope of Korea, The
#'      Progressive Party, Transition Korea).
#'    }
#'  \item \sQuote{party}: Factor with 7 levels. Denotes the political party of
#'  each member as of Sep. 2023. Note that independent members are assigned to
#'  their original party.
#'  \item \sQuote{gender}: Factor with 2 levels. \sQuote{M} (male) or \sQuote{F}
#'  (female).
#'  \item \sQuote{nelect}: Number of legislative terms in the assembly for each
#'  member. Ranges from 1 to 6.
#'  \item \sQuote{district}: Indicates if each member is a district
#'  representative (\code{TRUE}) or a proportional representative
#'  (\code{FALSE}).
#'  \item \sQuote{full}: Indicates if each member was in the assembly for the
#'  first 40 months. \code{TRUE} for the members in the office for all 40
#'  months. Members who started their term via by-election, resigned, or lost
#'  their seat for any reason during the 40 months are coded as \code{FALSE}.
#'  \item \sQuote{nbill}: Number of bills cosponsored by each member.
#' }
#'
#' Edge attribute: \sQuote{weight}. Given as a dissimilarity between two
#'   vertices. See the description above.
#'
#' @source The National Assembly of the Republic of Korea
#' \itemize{
#'  \item Bill information: https://likms.assembly.go.kr/bill/main.do
#'  \item Member information: https://open.assembly.go.kr/portal/assm/search/memberSchPage.do
#' }
#'
#' @references S. Kang and H.-S. Oh. On a notion of graph centrality based on
#'   \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} data depth.
#'   \emph{Journal of the American Statistical Association}, 121(553): 400--412, 2026.
NULL

## usethis namespace: start
#' @importFrom foreach %dopar%
#' @importFrom graphics lines
#' @importFrom graphics text
#' @importFrom igraph distances
#' @importFrom Rcpp sourceCpp
#' @importFrom stats aggregate
#' @importFrom stats cmdscale
#' @importFrom stats dist
#' @importFrom stats quantile
#' @importFrom stats rnorm
#' @importFrom utils flush.console
#' @importFrom withr local_par
#' @useDynLib L1centrality, .registration = TRUE
## usethis namespace: end
NULL

#' L1centrality: Graph/Network Analysis Based on \out{<i>L</i><sub>1</sub>}
#' Centrality
#'
#' Analyze graph/network data using \out{<i>L</i><sub>1</sub>} centrality and
#' prestige. Functions for deriving global, local, and group
#' \out{<i>L</i><sub>1</sub>} centrality/prestige are provided. Routines for
#' visual inspection of a graph/network are also provided. Details are in Kang
#' and Oh (2026a,b) and Kang (2025).
#'
#' Every function inside this package supports a distinct variety of graphs.
#' Edge weights can be considered by all functions. Additionally, it is
#' assumed that the graph to be analyzed is a connected graph in the case of an
#' undirected graph, or a strongly connected graph in the case of a directed
#' graph.
#'
#' Certain functions exclusively accommodate undirected graphs without vertex
#' multiplicity, while others support both directed and
#' undirected graphs with vertex multiplicities. The following table provides a
#' concise overview of the support range for each function.
#'
#' \tabular{rccc}{
#'    \strong{Functions} \tab \strong{Undirected graph} \tab \strong{Directed graph} \tab \strong{Vertex multiplicity}\cr
#'    [L1cent()], [L1centNB()], [L1centLOC()], [L1centGROUP()] \tab O \tab O \tab O \cr
#'    [L1centEDGE()] \tab O \tab X \tab O \cr
#'    [L1centMDS()] \tab O \tab X \tab X
#' }
#'
#' @references S. Kang. \emph{Topics in Non-Euclidean Dimension Reduction}. PhD
#'   thesis, Seoul National University, 2025.
#'
#'   S. Kang and H.-S. Oh. On a notion of graph centrality based on
#'   \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} data depth.
#'   \emph{Journal of the American Statistical Association}, 121(553): 400--412, 2026a.
#'
#'   S. Kang and H.-S. Oh.
#'   \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prominence measures
#'   for directed graphs. \emph{The American Statistician}, 80(2): 301--309, 2026b.
#'
#' @docType package
#' @name L1centrality-package
#' @aliases L1centrality-package L1centrality
#' @keywords internal
"_PACKAGE"

#' Marvel Cinematic Universe Movie Network
#'
#' @encoding UTF-8
#' @description
#'
#' Network between 32 movies from the Marvel Cinematic Universe (MCU) that were
#' released between 2008 and 2023. Each movie represents one vertex.
#'
#' An edge between movies
#' \ifelse{html}{\out{<i>i</i>}}{\eqn{i}} and
#' \ifelse{html}{\out{<i>j</i>}}{\eqn{j}} is formed if there is at least one
#' cast in common. Denoting the set of casts of movie \ifelse{html}{\out{<i>i</i>}}{\eqn{i}} as
#' \ifelse{html}{\out{<i>A<sub>i</sub></i>}}{\eqn{A_i}},
#' the weight of this edge is given as
#' \ifelse{html}{\out{(|<i>A<sub>i</sub></i>}\eqn{\cap}\out{<i>A<sub>j</sub></i>|/|<i>A<sub>i</sub></i>}\eqn{\cup}\out{<i>A<sub>j</sub></i>|)<sup>-1</sup>}}{\eqn{(|A_i\cap A_j|/|A_i\cup A_j|)^{-1}}},
#' where \eqn{|\cdot|} denotes the cardinality of a set.
#'
#' @name MCUmovie
#' @docType data
#' @keywords datasets
#' @usage data(MCUmovie)
#' @format An undirected, connected, and (edge) weighted \code{igraph} graph object with 32
#'   vertices and 278 edges.
#'
#'   Vertex attributes:
#'   \itemize{
#'    \item \sQuote{name}: name of the movie. e.g., \emph{Guardians
#'   of the Galaxy Vol. 3}.
#'    \item \sQuote{worldwidegross}: worldwide gross in USD. Archived from IMDb
#'    on Nov. 3rd, 2023.
#'    \item \sQuote{year}: release year of the movie.
#'   }
#'
#'   Edge attribute: \sQuote{weight}. Given as a dissimilarity between two
#'   vertices. See the description above.
#'
#' @source IMDb: https://www.imdb.com
#' @references G. Choi and H.-S. Oh. Heavy-snow transform: A new method for
#'   graph signals. Manuscript, 2021.
#'
#'   S. Kang and H.-S. Oh. On a notion of graph centrality based on
#'   \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} data depth.
#'   \emph{Journal of the American Statistical Association}, 121(553): 400--412, 2026.
NULL

#' @name L1centEDGE
#' @title Multiscale Edge Representation
#'
#' @description
#' Derives a multiscale edge representation. Each vertex is connected to its
#' local median, which is found in its
#' \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality-based
#' neighborhood.
#'
#' @details
#' In a global perspective, any given undirected graph can be represented as a
#' star-shaped directed graph, with each vertex making a connection to the
#' median vertex. Based on this idea, an undirected graph can be represented as
#' a directed graph, with each vertex making a connection to the \emph{local}
#' median vertex. The local median vertex of, say, \eqn{v_i}, is defined as a
#' median vertex among the
#' \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality-based
#' neighborhood of \eqn{v_i}. By varying the level of locality, the given graph
#' can be visually inspected at multiple scales. Refer to Kang and Oh (2026) for
#' details.
#'
#' @note
#' The function is valid only for undirected and connected graphs.
#'
#' @inheritParams L1centLOC
#' @param g An \code{igraph} graph object or a distance matrix.
#' * When \code{g} is an \code{igraph} object, the graph must be undirected and connected.
#' * When \code{g} is a matrix, it should be symmetric and represent a distance matrix.
#' All entries of the distance matrix must be finite.
#' @return \code{L1centEDGE()} returns an object of class \code{L1centEDGE}. It
#'   is a list of \sQuote{edge lists}. The length of the list is equivalent to
#'   the length of \code{alpha}, and the names of the list are the values of
#'   \code{alpha}. The \code{i}th component of the list is a 2-column matrix,
#'   and each row defines one directed edge, i.e., it is an edge list. The
#'   second column is the local (level \code{alpha[i]}) median of the vertex at
#'   the first column. There may be more than one edge from each vertex, since
#'   there may be more than one local median.
#'
#'  `print.L1centEDGE()` prints the edge lists and returns them invisibly.
#'
#' @export
#' @seealso [L1cent()], [L1centNB()], [L1centLOC()].
#' @examples
#' library(igraph)
#' MCU_edge <- L1centEDGE(MCUmovie, vertex_weight = V(MCUmovie)$worldwidegross, alpha = 5/32)
#' plot(MCU_edge)
#' @references S. Kang and H.-S. Oh. On a notion of graph centrality based on
#'   \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} data depth.
#'   \emph{Journal of the American Statistical Association}, 121(553): 400--412, 2026.
L1centEDGE <- function(g, vertex_weight, alpha, edge_weight_transform) UseMethod("L1centEDGE")

#' @name L1centEDGE
#' @exportS3Method L1centEDGE igraph
L1centEDGE.igraph <- function(g, vertex_weight=NULL, alpha, edge_weight_transform = NULL){
  validate_igraph(g)

  new_weight <- edge_weight_transform0(g, edge_weight_transform)
  if(!is.null(new_weight)) igraph::E(g)$weight <- new_weight
  D <- igraph::distances(g)
  attr(D, "label.igraph") <- igraph::V(g)$label
  attr(D, "color.igraph") <- igraph::V(g)$color
  L1centEDGE.matrix(D, vertex_weight=vertex_weight, alpha)
}

#' @name L1centEDGE
#' @exportS3Method L1centEDGE matrix
L1centEDGE.matrix <- function(g, vertex_weight=NULL, alpha, edge_weight_transform = NULL){
  eta <- vertex_weight
  if(is.null(eta)) eta <- rep(1,ncol(g))
  validate_matrix(g, eta)
  if(!all(alpha >= 0 & alpha <= 1))
    stop("alpha is not in the correct range: [0,1]")

  calls <- sys.calls()
  fnames <- sapply(calls, function(cl) as.character(cl[[1]]))
  # print(fnames)
  if(all(fnames[1:2] == c("L1centEDGE", "L1centEDGE.matrix")))
    message("DISTANCE matrix received")

  n <- ncol(g)
  label <- colnames(g)
  if(is.null(label))
    label <- 1:n

  if(identical(alpha, 1) | identical(alpha, 1L)){
    glob.median <- which(L1cent(g, eta) == 1)
    glob.median.length <- length(glob.median)
    glob.edgelist <- cbind(rep(1:n,each=glob.median.length),rep(glob.median,times=glob.median.length))
    rownames(glob.edgelist) <- NULL
    glob.edgelist <- matrix(label[glob.edgelist],ncol=2)
    glob.edgelist <- list(glob.edgelist)
    names(glob.edgelist) <- alpha
    return(structure(glob.edgelist,
                     class = "L1centEDGE",
                     alpha = alpha,
                     label = label,
                     label.igraph = attr(g,"label.igraph"),
                     color.igraph = attr(g,"color.igraph")))
  }

  # m <- ceiling(n*alpha)
  NB <- L1centNB(g)
  edgelist <- vector("list", length = length(alpha))
  names(edgelist) <- alpha

  for (i in seq_along(alpha)) {
    nb.index <-
      lapply(NB, function(l)
        which(l >= stats::quantile(l, 1 - alpha[i], type = 1)))
    loc.median <-
      lapply(1:length(nb.index), function(j){
        distsum <- colSums(g[nb.index[[j]],nb.index[[j]],drop=FALSE] * eta[nb.index[[j]]])
        nb.index[[j]][which(distsum == min(distsum))]})
    loc.median.length <- sapply(loc.median, length)
    edgelist[[i]] <- cbind(rep(1:n, times = loc.median.length), unlist(loc.median))
    rownames(edgelist[[i]]) <- NULL
    edgelist[[i]] <- matrix(label[edgelist[[i]]],ncol=2)
  }
  return(structure(edgelist,
                   class = "L1centEDGE",
                   alpha = alpha,
                   label = label,
                   label.igraph = attr(g,"label.igraph"),
                   color.igraph = attr(g,"color.igraph")))
}

#' @name L1centEDGE
#' @aliases print.L1centEDGE
#'
#' @param x An \code{L1centEDGE} object, obtained as a result of the function
#'   \code{L1centEDGE()}.
#' @param ... Further arguments passed to or from other methods.
#'   * `plot()` method: Further graphical parameters supplied to the internal
#'   [igraph::plot.igraph()] function. See [igraph::plot.common] document.
#'   * `print()` and `summary()` methods: Further arguments passed to the
#'   [base::print()] and [base::summary()] functions, respectively.
#' @export
print.L1centEDGE <- function(x, ...){
  for(i in seq_along(x)){
    cat("edge list at alpha = ",
        round(attr(x, "alpha")[[i]], 4), ":\n", sep = "")
    print.default(x[[i]], ...)
    if(i != length(x)) cat("\n")
  }
  return(invisible(x))
}


#' @name L1centEDGE
#' @aliases plot.L1centEDGE
#'
#' @return `plot.L1centEDGE()` draws directed graphs corresponding to each value
#'   of \code{alpha}. In each display, each vertex gives a directed edge to its
#'   local median vertex. The local median vertices are shown with larger
#'   circles.
#' @export
plot.L1centEDGE <- function(x, ...){
  args <- list(...)
  args.orignal <- args
  for(i in seq_along(x)){
    x[[i]] <- x[[i]][x[[i]][,1] != x[[i]][,2],]
    gi <- igraph::graph_from_edgelist(x[[i]])

    if(is.null(args.orignal$main)) args$main <- paste0("alpha = ", attr(x,"alpha")[i])
    if(is.null(args.orignal$edge.arrow.size)) args$edge.arrow.size <- 0.5
    if(is.null(args.orignal$vertex.size)) args$vertex.size <- sapply(if(is.null(igraph::V(gi)$name)) as.numeric(igraph::V(gi)) else igraph::V(gi)$name, function(elem) ifelse(elem %in% x[[i]][,2], 25, 15))
    if(is.null(args.orignal$vertex.label.family)) args$vertex.label.family <- "sans"
    if(is.null(args.orignal$vertex.label.cex)) args$vertex.label.cex <- 1
    if(is.null(args.orignal$vertex.label)) args$vertex.label <- if(is.null(attr(x, "label.igraph"))) igraph::V(gi)$name else attr(x, "label.igraph")[match(igraph::V(gi)$name, attr(x, "label"))]
    if(is.null(args.orignal$vertex.color)) args$vertex.color <- if(is.null(attr(x, "color.igraph"))) "gray" else attr(x, "color.igraph")[match(igraph::V(gi)$name, attr(x, "label"))]

    if(i > 1) withr::local_par(ask = TRUE)
    do.call(igraph::plot.igraph, c(list(gi), args))
  }
}

## igraph
validate_igraph <- function(g, checkdir = TRUE){
  if (checkdir & igraph::is_directed(g))
    stop("Directed graphs are not supported")
  if (!igraph::is_connected(g, mode = "strong"))
    stop(paste(strwrap("The graph is not strongly connected; Delete isolated vertices using 'igraph::delete_vertices()' or find the connected components of a graph using 'igraph::components()'", width = getOption("width")), collapse = "\n"))
}

## distance matrix
validate_matrix <- function(g, eta, checkdir = TRUE){
  if (checkdir & !isSymmetric.matrix(g))
    stop("Distance matrix is non-symmetric")
  if(any(is.infinite(g)))
    stop(paste(strwrap("The graph is not strongly connected; Delete isolated vertices using 'igraph::delete_vertices()' or find the connected components of a graph using 'igraph::components()'", width = getOption("width")), collapse = "\n"))
    # stop("Disconnected graphs are not supported; Delete isolated vertices using 'igraph::delete_vertices()' or find the connected components of a graph using 'igraph::components()'")
  if(length(eta) != ncol(g))
    stop("Length of eta differs from the number of vertices")
  if(any(eta < 0))
    stop("Entries of eta must be non-negative")
  if(sum(eta) <= 0)
    stop("sum(eta) must be positive")
}

## edge weight transformation
edge_weight_transform0 <- function(g, weight_transform){
  new_weight <- NULL
  if(is.null(weight_transform)){
    new_weight <- igraph::E(g)$weight
  }else if(!is.null(igraph::E(g)$weight)){
    new_weight <- weight_transform(igraph::E(g)$weight)
    if(length(new_weight) != length(igraph::E(g)$weight))
      stop("Length of edge weights after weight_transform differs from the original edge weight length")
  }
  return(new_weight)
}

#' @name L1centNB
#' @aliases L1presNB
#' @title L1 Centrality/Prestige-Based Neighborhood
#'
#' @description
#' Derives \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality- or
#' \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige-based
#' neighborhood of each vertex. For undirected graphs, the two neighborhood are
#' identical.
#'
#' @details
#' For an undirected graph, if the graph is symmetrized (in a way defined in Kang and Oh 2026a)
#' w.r.t. a vertex \ifelse{html}{\out{<i>v</i>}}{\eqn{v}}, vertex
#' \ifelse{html}{\out{<i>v</i>}}{\eqn{v}} becomes the graph median (Kang and Oh
#' 2026a), i.e., \ifelse{html}{\out{<i>v</i>}}{\eqn{v}} has
#' \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality 1. Based on
#' this property, we define the
#' \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality-based
#' neighborhood of vertex \ifelse{html}{\out{<i>v</i>}}{\eqn{v}} as vertices
#' that have large \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
#' centrality on the symmetrized graph w.r.t. vertex
#' \ifelse{html}{\out{<i>v</i>}}{\eqn{v}}.
#'
#' For a directed graph, a vertex of interest, say
#' \ifelse{html}{\out{<i>v</i>}}{\eqn{v}}, is made to a centrality and prestige
#' median vertex by the procedure described in Kang and Oh (2026b). We call the
#' resulting graph as the modified graph w.r.t.
#' \ifelse{html}{\out{<i>v</i>}}{\eqn{v}}.
#' \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
#' centrality (prestige) -based neighborhood of vertex
#' \ifelse{html}{\out{<i>v</i>}}{\eqn{v}} is a set of vertices that have large
#' \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality (prestige)
#' on the modified graph w.r.t. vertex
#' \ifelse{html}{\out{<i>v</i>}}{\eqn{v}}.
#'
#' @note
#' The function is valid only for connected graphs. If the graph is directed, it
#' must be strongly connected.
#'
#' @inheritParams L1cent
#' @return \code{L1centNB()} returns an object of class \code{L1centNB}. It
#'   is a list of numeric vectors. The length of the list is
#'   equivalent to the number of vertices in the graph \code{g}, and the names of the
#'   list are vertex names. Each component of the list is a numeric vector whose
#'   length is equivalent to the number of vertices in the graph \code{g}.
#'   Specifically, the \code{i}th component of the list is a vector of the
#'   \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality of each
#'   vertex, for the modified graph \code{g}
#'   w.r.t. the \code{i}th vertex (if \code{mode = "centrality"}) or the
#'   \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige of each
#'   vertex, for the modified graph \code{g}
#'   w.r.t. the \code{i}th vertex (if \code{mode = "prestige"}).
#'
#'  `print.L1centNB()` prints
#'  \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality or
#'  \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige values at
#'  the modified graph w.r.t. each vertex and returns them invisibly.
#'
#' @export
#' @seealso [L1cent()] for
#'   \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality/prestige,
#'   [L1centLOC()] and [L1centEDGE()] internally uses \code{L1centNB()}.
#
#' @examples
#' NB <- L1centNB(MCUmovie, vertex_weight = igraph::V(MCUmovie)$worldwidegross)
#' # Top 6 L1 centrality-based neighbors of "Iron Man"
#' utils::head(sort(NB$"Iron Man", decreasing = TRUE))
#' @references S. Kang and H.-S. Oh. On a notion of graph centrality based on
#'   \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} data depth.
#'   \emph{Journal of the American Statistical Association}, 121(553): 400--412, 2026a.
#'
#'   S. Kang and H.-S. Oh.
#'   \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prominence measures
#'   for directed graphs. \emph{The American Statistician}, 80(2): 301--309, 2026b.
L1centNB <- function(g, vertex_weight, mode, edge_weight_transform) UseMethod("L1centNB")

#' @name L1centNB
#' @exportS3Method L1centNB igraph
L1centNB.igraph <- function(g, vertex_weight = NULL, mode = c("centrality", "prestige"), edge_weight_transform = NULL){
  validate_igraph(g, checkdir = FALSE)
  mode <- match.arg(tolower(mode), choices = c("centrality", "prestige"))

  new_weight <- edge_weight_transform0(g, edge_weight_transform)
  if(!is.null(new_weight)) igraph::E(g)$weight <- new_weight
  D <- igraph::distances(g, mode = "out")
  L1centNB.matrix(D, vertex_weight, mode = mode)
}

#' @name L1centNB
#' @exportS3Method L1centNB matrix
L1centNB.matrix <- function(g, vertex_weight = NULL, mode = c("centrality", "prestige"), edge_weight_transform = NULL){
  eta <- vertex_weight
  if(is.null(eta)) eta <- rep(1,ncol(g))
  validate_matrix(g, eta, checkdir = FALSE)
  mode <- match.arg(tolower(mode), choices = c("centrality", "prestige"))

  calls <- sys.calls()
  fnames <- sapply(calls, function(cl) as.character(cl[[1]]))
  # print(fnames)
  if(all(fnames[1:2] == c("L1centNB", "L1centNB.matrix")))
    message("DISTANCE matrix received")

  n <- ncol(g)
  sg <- ifelse(isSymmetric.matrix(g), 1,
               min((g/t(g))[upper.tri(g) | lower.tri(g)]))
  label <- colnames(g)
  etasum <- sum(eta)
  NB <- lapply(1:n, function(i){
    neweta <- eta
    neweta[i] <- etasum/sg + neweta[i]
    c(L1cent(g,vertex_weight=neweta,mode=mode))
  })
  names(NB) <- label
  structure(NB,
            class = "L1centNB",
            mode = mode)
}

#' @name L1centNB
#' @aliases print.L1centNB
#'
#' @param x An \code{L1centNB} object, obtained as a result of the function
#'   \code{L1cent()}.
#' @param ... Further arguments passed to or from other methods. This argument
#'   is ignored here.
#' @export
print.L1centNB <- function(x, ...){
  if(is.null(names(x))) names(x) <- paste0("V",seq_along(x))
  for(i in seq_along(x)){
    cat("L1 ", attr(x, "mode"), " in the modified graph w.r.t. ",
        sQuote(names(x)[[i]]), ":\n", sep = "")
    print.default(c(x[[i]]), ...)
    if(i != length(x)) cat("\n")
  }
  return(invisible(x))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/L1centMDS.R
\name{L1centMDS}
\alias{L1centMDS}
\alias{L1centMDS.igraph}
\alias{L1centMDS.matrix}
\alias{plot.L1centMDS}
\alias{print.L1centMDS}
\title{Fitting a Target Plot}
\usage{
L1centMDS(g, tol, maxiter, verbose, edge_weight_transform)

\method{L1centMDS}{igraph}(
  g,
  tol = 1e-05,
  maxiter = 1000,
  verbose = TRUE,
  edge_weight_transform = NULL
)

\method{L1centMDS}{matrix}(
  g,
  tol = 1e-05,
  maxiter = 1000,
  verbose = TRUE,
  edge_weight_transform = NULL
)

\method{plot}{L1centMDS}(x, zoom = 1, main = NULL, show_edge = FALSE, ...)

\method{print}{L1centMDS}(x, ...)
}
\arguments{
\item{g}{An \code{igraph} graph object or a distance matrix.
\itemize{
\item When \code{g} is an \code{igraph} object, the graph must be undirected and connected.
\item When \code{g} is a matrix, it should be symmetric and represent a distance matrix.
All entries of the distance matrix must be finite.
}}

\item{tol}{A numerical tolerance. The gradient descent method terminates if
the relative magnitude of the gradient falls below \code{tol} as in Kruskal
(1964b). By default set to \ifelse{html}{\out{10<sup>-5</sup>}}{\eqn{10^{-5}}}.}

\item{maxiter}{A number of maximum iteration allowances for the gradient descent
algorithm. By default set to 1000.}

\item{verbose}{A boolean.
\itemize{
\item \code{TRUE} (the default): for each iteration, prints (1) current number of
iterations, (2) current stress, and (3) relative magnitude of the gradient
to the console. At the end, the final message is printed to the console; total
number of iterations and final stress.
\item \code{FALSE}: suppress message to the console.
}}

\item{edge_weight_transform}{An optional function to transform the edge weights
when \code{g} is an \code{igraph} object and an edge weight attribute exists. This
argument is ignored when \code{g} is a distance matrix.}

\item{x}{An \code{L1centMDS} object, obtained as a result of the function
\code{L1centMDS()}.}

\item{zoom}{A numerical value on how much to zoom-in the plot. By default set
to 1 (no zoom).}

\item{main}{Title of the plot. If set to \code{NULL} (the default), the title
prints \dQuote{Target plot / Stress = X}.}

\item{show_edge}{A boolean.
\itemize{
\item \code{TRUE}: edges are shown in the target plot. When \code{x}, the \code{L1centMDS}
object, is generated from a matrix object \code{g}, this argument is ignored
since \code{g} does not carry information about the edge connections. That is, edges
can only be drawn if the \code{L1centMDS} object is generated from an \code{igraph} object.
\item \code{FALSE} (the default): edges are not shown in the target plot.
}}

\item{...}{Further arguments passed to or from other methods.
\itemize{
\item \code{plot()} method: Further graphical parameters supplied to the internal
\code{\link[base:plot]{base::plot()}} (for points), \code{\link[graphics:text]{graphics::text()}} (for labels), and
\code{\link[graphics:segments]{graphics::segments()}} (for edges) functions. See \code{\link[graphics:par]{graphics::par()}}
document. To supply an argument to the first one, use the prefix
\sQuote{\code{plot.}}, for the second one, \sQuote{\code{text.}}, and for
the last one, \sQuote{\code{edge.}}. For instance, \code{plot.cex = 1} sets
the size of the point, whereas \code{text.cex = 1} sets the size of the
label.
\item \code{print()} method: This argument is ignored.
}}
}
\value{
\code{L1centMDS()} returns an object of class \code{L1centMDS}. It is a list
consisting of three vectors:
\itemize{
\item \sQuote{radius}: Radius of a point representing each vertex in the
target plot's circular coordinate system, i.e., \eqn{-\log(L_1\text{
 centrality})} for each vertex.
\item \sQuote{theta}: Angle (in radians) of a point representing each vertex
in the target plot's circular coordinate system.
\item \sQuote{stress}: Stress measure defined in Kruskal (1964a).
}

\code{plot.L1centMDS()} draws a target plot. Four concentric circles denote
the 1st to 4th quartiles of the radius, and the values of the
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality quartiles
are shown in red text. Note that red texts denote the
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality quartiles,
\emph{not} radius quartiles.

\code{print.L1centMDS()} prints number of iterations it took to fit a target plot.
}
\description{
\code{L1centMDS()} and \code{plot.L1centMDS()} are used together to draw a
target plot, which is a target-shaped 2D plot that aids in the visual
inspection of an undirected graph using the
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality. See Kang
and Oh (2026) for a formal definition of a target plot.
}
\details{
Denoting the \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
centrality of vertex \eqn{i} as \eqn{c_i\in(0,1]}, a point representing that vertex is placed
on a concentric circle with radius \eqn{r_i =
-\log(c_i)}. Representing each vertex as \eqn{(r_i, \theta_i)} (in circular
coordinates), the values of \eqn{\theta_i} are derived using nonmetric
multidimensional scaling proposed in Kruskal (1964a,b). The initial
configuration is derived using classical multidimensional scaling
(\code{\link[stats:cmdscale]{stats::cmdscale()}}). A gradient descent algorithm is employed in deriving
optimal \eqn{\theta_i}s.
}
\note{
The function \code{L1centMDS()} is valid only for undirected and connected
graphs. Also, \code{L1centMDS()} only considers graphs with equal vertex
multiplicities.
}
\examples{
parameters <- L1centMDS(MCUmovie, verbose = FALSE)
plot(parameters)
}
\references{
S. Kang and H.-S. Oh. On a notion of graph centrality based on
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} data depth.
\emph{Journal of the American Statistical Association}, 121(553): 400--412, 2026.

J. B. Kruskal. Multidimensional scaling by optimizing goodness of fit to a
nonmetric hypothesis. \emph{Psychometrika}, 29(1): 1--27, 1964a.

J. B. Kruskal. Nonmetric multidimensional scaling: a numerical method.
\emph{Psychometrika}, 29(2): 115--129, 1964b.
}
\seealso{
\code{\link[=L1cent]{L1cent()}} for
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality/prestige,
\code{\link[MASS:isoMDS]{MASS::isoMDS()}} and \code{\link[stats:cmdscale]{stats::cmdscale()}} for multidimensional scaling
methods.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MCUmovie.R
\docType{data}
\encoding{UTF-8}
\name{MCUmovie}
\alias{MCUmovie}
\title{Marvel Cinematic Universe Movie Network}
\format{
An undirected, connected, and (edge) weighted \code{igraph} graph object with 32
vertices and 278 edges.

Vertex attributes:
\itemize{
\item \sQuote{name}: name of the movie. e.g., \emph{Guardians
of the Galaxy Vol. 3}.
\item \sQuote{worldwidegross}: worldwide gross in USD. Archived from IMDb
on Nov. 3rd, 2023.
\item \sQuote{year}: release year of the movie.
}

Edge attribute: \sQuote{weight}. Given as a dissimilarity between two
vertices. See the description above.
}
\source{
IMDb: https://www.imdb.com
}
\usage{
data(MCUmovie)
}
\description{
Network between 32 movies from the Marvel Cinematic Universe (MCU) that were
released between 2008 and 2023. Each movie represents one vertex.

An edge between movies
\ifelse{html}{\out{<i>i</i>}}{\eqn{i}} and
\ifelse{html}{\out{<i>j</i>}}{\eqn{j}} is formed if there is at least one
cast in common. Denoting the set of casts of movie \ifelse{html}{\out{<i>i</i>}}{\eqn{i}} as
\ifelse{html}{\out{<i>A<sub>i</sub></i>}}{\eqn{A_i}},
the weight of this edge is given as
\ifelse{html}{\out{(|<i>A<sub>i</sub></i>}\eqn{\cap}\out{<i>A<sub>j</sub></i>|/|<i>A<sub>i</sub></i>}\eqn{\cup}\out{<i>A<sub>j</sub></i>|)<sup>-1</sup>}}{\eqn{(|A_i\cap A_j|/|A_i\cup A_j|)^{-1}}},
where \eqn{|\cdot|} denotes the cardinality of a set.
}
\references{
G. Choi and H.-S. Oh. Heavy-snow transform: A new method for
graph signals. Manuscript, 2021.

S. Kang and H.-S. Oh. On a notion of graph centrality based on
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} data depth.
\emph{Journal of the American Statistical Association}, 121(553): 400--412, 2026.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/L1centGROUP.R
\name{L1centGROUP}
\alias{L1centGROUP}
\alias{L1presGROUP}
\alias{L1centGROUP.igraph}
\alias{L1centGROUP.matrix}
\alias{print.L1centGROUP}
\title{Group L1 Centrality/Prestige}
\usage{
L1centGROUP(g, nodes, vertex_weight, mode, method, edge_weight_transform)

\method{L1centGROUP}{igraph}(
  g,
  nodes,
  vertex_weight = NULL,
  mode = c("centrality", "prestige"),
  method = c("minimum", "maximum", "average"),
  edge_weight_transform = NULL
)

\method{L1centGROUP}{matrix}(
  g,
  nodes,
  vertex_weight = NULL,
  mode = c("centrality", "prestige"),
  method = "minimum",
  edge_weight_transform = NULL
)

\method{print}{L1centGROUP}(x, ...)
}
\arguments{
\item{g}{An \code{igraph} graph object or a distance matrix.
\itemize{
\item When \code{g} is an \code{igraph} object, the graph must be connected.
For a directed graph, it must be strongly connected.
\item When \code{g} is a matrix, it should represent a distance matrix.
All entries of the distance matrix must be finite. Here, the
\ifelse{html}{\out{(<i>i,j</i>)}}{\eqn{(i,j)}} component of the distance
matrix is the geodesic distance from the
\ifelse{html}{\out{<i>i</i>}}{\eqn{i}}th vertex to the
\ifelse{html}{\out{<i>j</i>}}{\eqn{j}}th vertex.
}}

\item{nodes}{A vector of integers or a list of integer vectors. Each integer
indicates the index of the vertex. When the argument is provided as a list,
the function computes the group
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prominence for each
set of vertices specified by the individual list elements. When it is given
as an integer vector, the function computes the group
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prominence for the
single group defined by that vector.}

\item{vertex_weight}{An optional nonnegative multiplicity (weight) vector for (vertex)
weighted networks. The sum of its components must be positive. If set to
\code{NULL} (the default), all vertices will have the same positive weight
(multiplicity) of 1, i.e., \code{g} is treated as a vertex unweighted graph. The
length of the \code{vertex_weight} must be equivalent to the number of vertices.}

\item{mode}{A character string. For an undirected graph, either choice gives
the same result.
\itemize{
\item \code{centrality} (the default): \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
centrality (prominence of each vertex in terms of \emph{making} a choice) is
used for analysis.
\item \code{prestige}: \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
prestige (prominence of each vertex in terms of \emph{receiving} a choice)
is used for analysis.
}}

\item{method}{A character string. It specifies the method of setting the edge
weight between the pseudo-vertex and the other vertices. Note that the S3
method for the \code{matrix} class only supports the \code{minimum} option. This
is because it is not possible to derive the group reduced graph's distance
matrix from the original distance matrix when using the maximum or average
method. On the other hand, the group reduced graph's distance matrix can be
derived from the original distance matrix when the minimum method is used.
See the discussion in Kang (2025).
\itemize{
\item \code{minimum} (the default): the minimum method is used in setting the edge weights.
\item \code{maximum}: the maximum method is used in setting the edge weights.
\item \code{average}: the average method is used in setting the edge weights.
}}

\item{edge_weight_transform}{An optional function to transform the edge weights
when \code{g} is an \code{igraph} object and an edge weight attribute exists. This
argument is ignored when \code{g} is a distance matrix.}

\item{x}{An \code{L1centGROUP} object, obtained as a result of the function
\code{L1centGROUP()}.}

\item{...}{Further arguments passed to or from other methods. This argument
is ignored here.}
}
\value{
\code{L1centGROUP()} returns an object of class \code{L1centGROUP}. It is a
numeric value of the group
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality (if
\code{mode = "centrality"}) or the group
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige (if
\code{mode = "prestige"}) of the specified group of vertices.

\code{print.L1centGROUP()} prints group
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality or group
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige value and
returns it invisibly.
}
\description{
Computes group \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
centrality or group \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
prestige for the specified groups of vertices. For undirected graphs, the two
measures are identical.
}
\details{
Given a group of vertices on a graph, we first construct a group reduced
graph by replacing the group of vertices by a single \sQuote{pseudo-vertex}
(see \code{\link[=group_reduce]{group_reduce()}} for the method of setting vertex multiplicities and
edge weights in the group reduced graph). The group
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality (prestige)
of this group is defined as the
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality (prestige)
of the pseudo-vertex in the group reduced graph.
}
\note{
The function is valid only for connected graphs. If the graph is directed, it
must be strongly connected. Multiple edges (edges with the same head and tail
vertices) are not allowed, because they make the edge weight setting procedure
confusing.
}
\examples{
# Group L1 centrality of the 'Spider-Man' series
vertex_weight <- igraph::V(MCUmovie)$worldwidegross
L1centGROUP(MCUmovie, nodes = list(16, c(16,23,27)), vertex_weight = vertex_weight)
}
\references{
S. Kang. \emph{Topics in Non-Euclidean Dimension Reduction}. PhD thesis,
Seoul National University, 2025.
}
\seealso{
\code{\link[=L1cent]{L1cent()}} for \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
centrality/prestige, \code{\link[=group_reduce]{group_reduce()}} for details on the minimum, maximum, and average methods.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/L1centNB.R, R/summary.R
\name{L1centNB}
\alias{L1centNB}
\alias{L1presNB}
\alias{L1centNB.igraph}
\alias{L1centNB.matrix}
\alias{print.L1centNB}
\alias{summary.L1centNB}
\title{L1 Centrality/Prestige-Based Neighborhood}
\usage{
L1centNB(g, vertex_weight, mode, edge_weight_transform)

\method{L1centNB}{igraph}(
  g,
  vertex_weight = NULL,
  mode = c("centrality", "prestige"),
  edge_weight_transform = NULL
)

\method{L1centNB}{matrix}(
  g,
  vertex_weight = NULL,
  mode = c("centrality", "prestige"),
  edge_weight_transform = NULL
)

\method{print}{L1centNB}(x, ...)

\method{summary}{L1centNB}(object, ...)
}
\arguments{
\item{g}{An \code{igraph} graph object or a distance matrix.
\itemize{
\item When \code{g} is an \code{igraph} object, the graph must be connected.
For a directed graph, it must be strongly connected.
\item When \code{g} is a matrix, it should represent a distance matrix.
All entries of the distance matrix must be finite. Here, the
\ifelse{html}{\out{(<i>i,j</i>)}}{\eqn{(i,j)}} component of the distance
matrix is the geodesic distance from the
\ifelse{html}{\out{<i>i</i>}}{\eqn{i}}th vertex to the
\ifelse{html}{\out{<i>j</i>}}{\eqn{j}}th vertex.
}}

\item{vertex_weight}{An optional nonnegative multiplicity (weight) vector for (vertex)
weighted networks. The sum of its components must be positive. If set to
\code{NULL} (the default), all vertices will have the same positive weight
(multiplicity) of 1, i.e., \code{g} is treated as a vertex unweighted graph. The
length of the \code{vertex_weight} must be equivalent to the number of vertices.}

\item{mode}{A character string. For an undirected graph, either choice gives
the same result.
\itemize{
\item \code{centrality} (the default): \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
centrality (prominence of each vertex in terms of \emph{making} a choice) is
used for analysis.
\item \code{prestige}: \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
prestige (prominence of each vertex in terms of \emph{receiving} a choice)
is used for analysis.
}}

\item{edge_weight_transform}{An optional function to transform the edge weights
when \code{g} is an \code{igraph} object and an edge weight attribute exists. This
argument is ignored when \code{g} is a distance matrix.}

\item{x}{An \code{L1centNB} object, obtained as a result of the function
\code{L1cent()}.}

\item{...}{Further arguments passed to or from other methods. This argument
is ignored here.}

\item{object}{An \code{L1centNB} object, obtained as a result of the function
\code{L1centNB()}.}
}
\value{
\code{L1centNB()} returns an object of class \code{L1centNB}. It
is a list of numeric vectors. The length of the list is
equivalent to the number of vertices in the graph \code{g}, and the names of the
list are vertex names. Each component of the list is a numeric vector whose
length is equivalent to the number of vertices in the graph \code{g}.
Specifically, the \code{i}th component of the list is a vector of the
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality of each
vertex, for the modified graph \code{g}
w.r.t. the \code{i}th vertex (if \code{mode = "centrality"}) or the
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige of each
vertex, for the modified graph \code{g}
w.r.t. the \code{i}th vertex (if \code{mode = "prestige"}).

\code{print.L1centNB()} prints
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality or
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige values at
the modified graph w.r.t. each vertex and returns them invisibly.

\code{summary.L1centNB()} returns an object of class \code{table}.
It is a summary of the prominence values with the five-number summary,
mean, and the Gini coefficient, at each modified graph.
}
\description{
Derives \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality- or
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige-based
neighborhood of each vertex. For undirected graphs, the two neighborhood are
identical.
}
\details{
For an undirected graph, if the graph is symmetrized (in a way defined in Kang and Oh 2026a)
w.r.t. a vertex \ifelse{html}{\out{<i>v</i>}}{\eqn{v}}, vertex
\ifelse{html}{\out{<i>v</i>}}{\eqn{v}} becomes the graph median (Kang and Oh
2026a), i.e., \ifelse{html}{\out{<i>v</i>}}{\eqn{v}} has
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality 1. Based on
this property, we define the
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality-based
neighborhood of vertex \ifelse{html}{\out{<i>v</i>}}{\eqn{v}} as vertices
that have large \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
centrality on the symmetrized graph w.r.t. vertex
\ifelse{html}{\out{<i>v</i>}}{\eqn{v}}.

For a directed graph, a vertex of interest, say
\ifelse{html}{\out{<i>v</i>}}{\eqn{v}}, is made to a centrality and prestige
median vertex by the procedure described in Kang and Oh (2026b). We call the
resulting graph as the modified graph w.r.t.
\ifelse{html}{\out{<i>v</i>}}{\eqn{v}}.
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
centrality (prestige) -based neighborhood of vertex
\ifelse{html}{\out{<i>v</i>}}{\eqn{v}} is a set of vertices that have large
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality (prestige)
on the modified graph w.r.t. vertex
\ifelse{html}{\out{<i>v</i>}}{\eqn{v}}.
}
\note{
The function is valid only for connected graphs. If the graph is directed, it
must be strongly connected.
}
\examples{
NB <- L1centNB(MCUmovie, vertex_weight = igraph::V(MCUmovie)$worldwidegross)
# Top 6 L1 centrality-based neighbors of "Iron Man"
utils::head(sort(NB$"Iron Man", decreasing = TRUE))
}
\references{
S. Kang and H.-S. Oh. On a notion of graph centrality based on
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} data depth.
\emph{Journal of the American Statistical Association}, 121(553): 400--412, 2026a.

S. Kang and H.-S. Oh.
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prominence measures
for directed graphs. \emph{The American Statistician}, 80(2): 301--309, 2026b.
}
\seealso{
\code{\link[=L1cent]{L1cent()}} for
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality/prestige,
\code{\link[=L1centLOC]{L1centLOC()}} and \code{\link[=L1centEDGE]{L1centEDGE()}} internally uses \code{L1centNB()}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/L1centLOC.R, R/summary.R
\name{L1centLOC}
\alias{L1centLOC}
\alias{L1presLOC}
\alias{L1centLOC.igraph}
\alias{L1centLOC.matrix}
\alias{print.L1centLOC}
\alias{plot.L1centLOC}
\alias{summary.L1centLOC}
\title{Local L1 Centrality/Prestige}
\usage{
L1centLOC(g, vertex_weight, alpha, mode, edge_weight_transform, parallel)

\method{L1centLOC}{igraph}(
  g,
  vertex_weight = NULL,
  alpha,
  mode = c("centrality", "prestige"),
  edge_weight_transform = NULL,
  parallel = FALSE
)

\method{L1centLOC}{matrix}(
  g,
  vertex_weight = NULL,
  alpha,
  mode = c("centrality", "prestige"),
  edge_weight_transform = NULL,
  parallel = FALSE
)

\method{print}{L1centLOC}(x, ...)

\method{plot}{L1centLOC}(x, y = NULL, add = FALSE, threshold = NULL, ...)

\method{summary}{L1centLOC}(object, ...)
}
\arguments{
\item{g}{An \code{igraph} graph object or a distance matrix.
\itemize{
\item When \code{g} is an \code{igraph} object, the graph must be connected.
For a directed graph, it must be strongly connected.
\item When \code{g} is a matrix, it should represent a distance matrix.
All entries of the distance matrix must be finite. Here, the
\ifelse{html}{\out{(<i>i,j</i>)}}{\eqn{(i,j)}} component of the distance
matrix is the geodesic distance from the
\ifelse{html}{\out{<i>i</i>}}{\eqn{i}}th vertex to the
\ifelse{html}{\out{<i>j</i>}}{\eqn{j}}th vertex.
}}

\item{vertex_weight}{An optional nonnegative multiplicity (weight) vector for (vertex)
weighted networks. The sum of its components must be positive. If set to
\code{NULL} (the default), all vertices will have the same positive weight
(multiplicity) of 1, i.e., \code{g} is treated as a vertex unweighted graph. The
length of the \code{vertex_weight} must be equivalent to the number of vertices.}

\item{alpha}{A number or a numeric vector of locality levels. Values
must be between 0 and 1.}

\item{mode}{A character string. For an undirected graph, either choice gives
the same result.
\itemize{
\item \code{centrality} (the default): \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
centrality (prominence of each vertex in terms of \emph{making} a choice) is
used for analysis.
\item \code{prestige}: \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
prestige (prominence of each vertex in terms of \emph{receiving} a choice)
is used for analysis.
}}

\item{edge_weight_transform}{An optional function to transform the edge weights
when \code{g} is an \code{igraph} object and an edge weight attribute exists. This
argument is ignored when \code{g} is a distance matrix.}

\item{parallel}{A boolean indicating whether to run the algorithm
parallel across multiple cores. By default set to \code{FALSE}.}

\item{x}{An \code{L1centLOC} object, obtained as a result of the function
\code{L1centLOC()}.}

\item{...}{Further arguments passed to or from other methods.}

\item{y}{An optional argument providing the coordinates for a scatter plot.
It could be an object of class \code{L1cent} or \code{L1centLOC}, or a
numeric vector.}

\item{add}{A logical value. This argument is considered only when drawing a Lorenz curve.
\itemize{
\item \code{TRUE}: add the Lorenz curve to an already existing plot.
\item \code{FALSE} (the default): draw the Lorenz curve to a new graphic device.
}}

\item{threshold}{A number between 0 and 1. Vertices that have their maximum
and minimum local \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
prominence value difference above the \code{threshold} are indicated in
colored lines.}

\item{object}{An \code{L1centLOC} object, obtained as a result of the function
\code{L1centLOC()}.}
}
\value{
\code{L1centLOC()} returns an object of class \code{L1centLOC}. It is
a list of numeric vectors. The length of the list is equivalent to the
length of \code{alpha}, and the names of the list are the values of
\code{alpha}. Each component of the list is a numeric vector whose length
is equivalent to the number of vertices in the graph \code{g}.
Specifically, the \code{i}th component of the list is a vector of local
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality at level
\code{alpha[i]} for each vertex (if \code{mode = "centrality"}) or local
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige at level
\code{alpha[i]} for each vertex (if \code{mode = "prestige"}).

\code{print.L1centLOC()} prints local
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality or local
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige values at
each locality level \code{alpha} and returns them invisibly.

\code{plot.L1centLOC()} draws a following plot.
\itemize{
\item \code{y} is not supplied and \code{alpha} is of length one: A Lorenz curve (the group heterogeneity plot)
and returns an invisible copy of a Gini coefficient (the group heterogeneity
index). \code{threshold} is ignored.
\item \code{y} is supplied and \code{alpha} is of length one: A scatter plot of
\code{x} versus \code{y}. \code{threshold} is ignored.
\item \code{alpha}'s length is larger than one: A plot of \code{alpha} versus
local \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prominence
values (in a uniform margin) for each vertex. If \code{threshold} is set,
vertices that have their maximum
and minimum local \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
prominence value difference above the \code{threshold} are indicated in
colored lines. \code{y} is ignored.
}

\code{summary.L1centLOC()} returns an object of class \code{table}.
It is a summary of the prominence values with the five-number summary,
mean, and the Gini coefficient, at each level of \code{alpha}.
}
\description{
Computes local \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
centrality or local \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
prestige at each \code{alpha} level for every vertex. For undirected graphs,
the two measures are identical.
}
\details{
Suppose that the given graph has \ifelse{html}{\out{<i>n</i>}}{\eqn{n}}
vertices. We choose about \eqn{n\alpha} vertices
(\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality- or
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige-based
neighborhood) for each vertex (see \code{\link[=L1centNB]{L1centNB()}}), and compute the
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality or
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige of the vertex
conditioned on these vertices, i.e., derive the
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality or
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige locally. For
details, refer to Kang and Oh (2026a) for undirected graphs, and Kang and Oh
(2026b) for directed graphs.
}
\note{
The function is valid only for connected graphs. If the graph is directed, it
must be strongly connected.
}
\examples{
weight <- igraph::V(MCUmovie)$worldwidegross
MCUmovie_cent <- L1cent(MCUmovie, vertex_weight = weight)
MCUmovie_loc_cent <- L1centLOC(MCUmovie, vertex_weight = weight, alpha = 5/32)
plot(MCUmovie_cent, MCUmovie_loc_cent,
     main = "MCU movie network: global vs. local centrality")
}
\references{
S. Kang and H.-S. Oh. On a notion of graph centrality based on
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} data depth.
\emph{Journal of the American Statistical Association}, 121(553): 400--412, 2026a.

S. Kang and H.-S. Oh.
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prominence measures
for directed graphs. \emph{The American Statistician}, 80(2): 301--309, 2026b.
}
\seealso{
\code{\link[=L1cent]{L1cent()}} for
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality/prestige,
\code{\link[=L1centNB]{L1centNB()}} for \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
centrality/prestige-based neighborhood.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rokassembly21.R
\docType{data}
\name{rokassembly21}
\alias{rokassembly21}
\title{Republic of Korea's 21st National Assembly Bill Cosponsorship Network}
\format{
An undirected, connected, and (edge) weighted \code{igraph} graph
object with 317 vertices and 47,657 edges.

Vertex attributes:
\itemize{
\item \sQuote{name}: Pseudonyms of each member. They are in the format of
the party's initial character, followed by a random number (e.g., D4). Each
party's initial character is:
\itemize{
\item \sQuote{D}: Democratic Party of Korea.
\item \sQuote{P}: People Power Party.
\item \sQuote{J}: Justice Party.
\item \sQuote{O}: Others (Basic Income Party, Hope of Korea, The
Progressive Party, Transition Korea).
}
\item \sQuote{party}: Factor with 7 levels. Denotes the political party of
each member as of Sep. 2023. Note that independent members are assigned to
their original party.
\item \sQuote{gender}: Factor with 2 levels. \sQuote{M} (male) or \sQuote{F}
(female).
\item \sQuote{nelect}: Number of legislative terms in the assembly for each
member. Ranges from 1 to 6.
\item \sQuote{district}: Indicates if each member is a district
representative (\code{TRUE}) or a proportional representative
(\code{FALSE}).
\item \sQuote{full}: Indicates if each member was in the assembly for the
first 40 months. \code{TRUE} for the members in the office for all 40
months. Members who started their term via by-election, resigned, or lost
their seat for any reason during the 40 months are coded as \code{FALSE}.
\item \sQuote{nbill}: Number of bills cosponsored by each member.
}

Edge attribute: \sQuote{weight}. Given as a dissimilarity between two
vertices. See the description above.
}
\source{
The National Assembly of the Republic of Korea
\itemize{
\item Bill information: https://likms.assembly.go.kr/bill/main.do
\item Member information: https://open.assembly.go.kr/portal/assm/search/memberSchPage.do
}
}
\usage{
data(rokassembly21)
}
\description{
Network between 317 members of the Republic of Korea's 21st National
Assembly (May 30th, 2020--May 29th, 2024). Each member of the assembly
represents one vertex.

An edge between two members is formed if there is at least one cosponsored
bill. The weight of this edge is given as 1/(number of cosponsored bills
between two members) during the first 40 months of the 21st assembly (Jun.
2020--Sep. 2023).
}
\references{
S. Kang and H.-S. Oh. On a notion of graph centrality based on
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} data depth.
\emph{Journal of the American Statistical Association}, 121(553): 400--412, 2026.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/L1cent.R, R/summary.R
\name{L1cent}
\alias{L1cent}
\alias{L1prestige}
\alias{L1pres}
\alias{L1cent.igraph}
\alias{L1cent.matrix}
\alias{print.L1cent}
\alias{plot.L1cent}
\alias{summary.L1cent}
\title{L1 Centrality/Prestige}
\usage{
L1cent(g, vertex_weight, mode, edge_weight_transform)

\method{L1cent}{igraph}(
  g,
  vertex_weight = NULL,
  mode = c("centrality", "prestige"),
  edge_weight_transform = NULL
)

\method{L1cent}{matrix}(
  g,
  vertex_weight = NULL,
  mode = c("centrality", "prestige"),
  edge_weight_transform = NULL
)

\method{print}{L1cent}(x, ...)

\method{plot}{L1cent}(x, y = NULL, add = FALSE, ...)

\method{summary}{L1cent}(object, ...)
}
\arguments{
\item{g}{An \code{igraph} graph object or a distance matrix.
\itemize{
\item When \code{g} is an \code{igraph} object, the graph must be connected.
For a directed graph, it must be strongly connected.
\item When \code{g} is a matrix, it should represent a distance matrix.
All entries of the distance matrix must be finite. Here, the
\ifelse{html}{\out{(<i>i,j</i>)}}{\eqn{(i,j)}} component of the distance
matrix is the geodesic distance from the
\ifelse{html}{\out{<i>i</i>}}{\eqn{i}}th vertex to the
\ifelse{html}{\out{<i>j</i>}}{\eqn{j}}th vertex.
}}

\item{vertex_weight}{An optional nonnegative multiplicity (weight) vector for (vertex)
weighted networks. The sum of its components must be positive. If set to
\code{NULL} (the default), all vertices will have the same positive weight
(multiplicity) of 1, i.e., \code{g} is treated as a vertex unweighted graph. The
length of the \code{vertex_weight} must be equivalent to the number of vertices.}

\item{mode}{A character string. For an undirected graph, either choice gives
the same result.
\itemize{
\item \code{centrality} (the default): \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
centrality (prominence of each vertex in terms of \emph{making} a choice) is
used for analysis.
\item \code{prestige}: \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
prestige (prominence of each vertex in terms of \emph{receiving} a choice)
is used for analysis.
}}

\item{edge_weight_transform}{An optional function to transform the edge weights
when \code{g} is an \code{igraph} object and an edge weight attribute exists. This
argument is ignored when \code{g} is a distance matrix.}

\item{x}{An \code{L1cent} object, obtained as a result of the function
\code{L1cent()}.}

\item{...}{Further arguments passed to or from other methods.}

\item{y}{An optional argument providing the coordinates for a scatter plot.
It could be an object of class \code{L1cent} or \code{L1centLOC}, or a
numeric vector.}

\item{add}{A logical value. This argument is considered only when drawing a Lorenz curve.
\itemize{
\item \code{TRUE}: add the Lorenz curve to an already existing plot.
\item \code{FALSE} (the default): draw the Lorenz curve to a new graphic device.
}}

\item{object}{An \code{L1cent} object, obtained as a result of the function
\code{L1cent()}.}
}
\value{
\code{L1cent()} returns an object of class \code{L1cent}. It is a
numeric vector whose length is equivalent to the number of vertices in the
graph \code{g}. Each component of the vector is the
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality (if
\code{mode = "centrality"}) or the
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige (if
\code{mode = "prestige"}) of each vertex in the given graph.

\code{print.L1cent()} prints
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality or
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige values and
returns them invisibly.

\code{plot.L1cent()} draws a Lorenz curve (the group heterogeneity plot)
and returns an invisible copy of a Gini coefficient (the group
heterogeneity index) if argument \code{y} is not supplied. Otherwise, it
draws a scatter plot.

\code{summary.L1cent()} returns an object of class \code{table}.
It is a summary of the prominence values with the five-number summary,
mean, and the Gini coefficient.
}
\description{
Computes \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality or
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige for each
vertex. The \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}
centrality/prestige is a graph centrality/prestige measure defined for the
vertices of a graph. It is (roughly) defined by (1 \eqn{-} minimum
multiplicity required for a selected vertex to become the median of the
graph). For directed graphs,
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality quantifies
the prominence of a vertex in \emph{making} a choice and
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige quantifies
the prominence of a vertex in \emph{receiving} a choice. For undirected graphs,
the two measures are identical.
}
\details{
Suppose that \code{g} is a (strongly) connected graph consisting of
\ifelse{html}{\out{<i>n</i>}}{\eqn{n}} vertices
\ifelse{html}{\out{<i>v</i><sub>1</sub>, ...,
<i>v<sub>n</sub></i>}}{{\eqn{v_1,\dots,v_n}}}
whose multiplicities (vertex weights) are \eqn{\eta_1,\dots,\eta_n \geq 0}, respectively,
and \eqn{\eta_{\cdot} = \sum_{k=1}^n \eta_k > 0}.

The centrality median vertex of this graph is the node minimizing the
weighted sum of distances. That is,
\ifelse{html}{\out{<i>v<sub>i</sub></i>}}{{\eqn{v_i}}} is the centrality
median vertex if
\deqn{
 \sum_{k=1}^{n} \eta_k d(v_i, v_k)
}
is minimized, where \eqn{d(v_i,v_k)} denotes the geodesic (shortest path)
distance from \eqn{v_i} to \eqn{v_k}. See \code{\link[igraph:distances]{igraph::distances()}} for
algorithms for computing geodesic distances between vertices. When the
indices are swapped to \eqn{d(v_k, v_i)} in the display above, we call the
node minimizing the weighted sum as the prestige median vertex. When the
graph is undirected, the prestige median vertex and the centrality median
vertex coincide, and we call it the graph median, following Hakimi (1964).

The \ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality for an
arbitrary node \ifelse{html}{\out{<i>v<sub>i</sub></i>}}{{\eqn{v_i}}} is
defined as \sQuote{one minus the minimum weight that is required to make it a
centrality median vertex.} This concept of centrality is closely related to the
data depth for ranking multivariate data, as defined in Vardi and Zhang
(2000). It turns out that the following formula computes the
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality for the
vertex \ifelse{html}{\out{<i>v<sub>i</sub></i>}}{{\eqn{v_i}}}:
\deqn{
 1-\mathcal{S}(\texttt{g})\max_{j\neq i}\left\{\frac{\sum_{k=1}^{n}\eta_k (d(v_i,v_k) - d(v_j,v_k)) }{\eta_{\cdot}d(v_j,v_i)}\right\}^{+},
}
where \eqn{\{\cdot\}^{+}=\max(\cdot,0)} and \eqn{\mathcal{S}(\texttt{g}) =
\min_{i\neq j} d(v_i,v_j)/d(v_j,v_i)}. The
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality of a vertex
is in \eqn{[0,1]} by the triangle inequality, and the centrality median
vertex has centrality 1. The
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige is defined
analogously, with the indices inside the distance function swapped.

For an undirected graph, \eqn{\mathcal{S}(\texttt{g}) = 1} since the distance
function is symmetric. Moreover,
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality and
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prestige measures concide.

For details, refer to Kang and Oh (2026a) for undirected graphs, and Kang and
Oh (2026b) for directed graphs.
}
\note{
The function is valid only for connected graphs. If the graph is directed, it
must be strongly connected.
}
\examples{
# igraph object and distance matrix as an input lead to the same result
vertex_weight <- igraph::V(MCUmovie)$worldwidegross
cent_igraph <- L1cent(MCUmovie, vertex_weight = vertex_weight)
cent_matrix <- L1cent(igraph::distances(MCUmovie), vertex_weight = vertex_weight)
all(cent_igraph == cent_matrix)

# Top 6 vertices with the highest L1 centrality
utils::head(sort(cent_igraph, decreasing = TRUE))

# Lorenz curve
plot(cent_igraph)

# Summary statistics
summary(cent_igraph)

}
\references{
S. L. Hakimi. Optimum locations of switching centers and the
absolute centers and medians of a graph. \emph{Operations Research},
12(3): 450--459, 1964.

S. Kang and H.-S. Oh. On a notion of graph centrality based on
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} data depth.
\emph{Journal of the American Statistical Association}, 121(553): 400--412, 2026a.

S. Kang and H.-S. Oh.
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prominence measures
for directed graphs. \emph{The American Statistician}, 80(2): 301--309, 2026b.

Y. Vardi and C.-H. Zhang. The multivariate
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}}-median and
associated data depth. \emph{Proceedings of the National Academy of Sciences},
97(4): 1423--1426, 2000.
}
\seealso{
\code{\link[=L1centLOC]{L1centLOC()}}, \code{\link[=L1centNB]{L1centNB()}}, \code{\link[=L1centMDS]{L1centMDS()}}, \code{\link[=L1centEDGE]{L1centEDGE()}},
\code{\link[=L1centGROUP]{L1centGROUP()}} for
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality- or
prestige-based analysis. See \link{L1centrality-package} for each function's
support range.

\code{\link[igraph:betweenness]{igraph::betweenness()}}, \code{\link[igraph:closeness]{igraph::closeness()}},
\code{\link[igraph:degree]{igraph::degree()}}, \code{\link[igraph:eigen_centrality]{igraph::eigen_centrality()}} for centrality measures.

\link{Heterogeneity} for drawing the Lorenz curve and computing the Gini
coefficient.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/L1centEDGE.R, R/summary.R
\name{L1centEDGE}
\alias{L1centEDGE}
\alias{L1centEDGE.igraph}
\alias{L1centEDGE.matrix}
\alias{print.L1centEDGE}
\alias{plot.L1centEDGE}
\alias{summary.L1centEDGE}
\title{Multiscale Edge Representation}
\usage{
L1centEDGE(g, vertex_weight, alpha, edge_weight_transform)

\method{L1centEDGE}{igraph}(g, vertex_weight = NULL, alpha, edge_weight_transform = NULL)

\method{L1centEDGE}{matrix}(g, vertex_weight = NULL, alpha, edge_weight_transform = NULL)

\method{print}{L1centEDGE}(x, ...)

\method{plot}{L1centEDGE}(x, ...)

\method{summary}{L1centEDGE}(object, ...)
}
\arguments{
\item{g}{An \code{igraph} graph object or a distance matrix.
\itemize{
\item When \code{g} is an \code{igraph} object, the graph must be undirected and connected.
\item When \code{g} is a matrix, it should be symmetric and represent a distance matrix.
All entries of the distance matrix must be finite.
}}

\item{vertex_weight}{An optional nonnegative multiplicity (weight) vector for (vertex)
weighted networks. The sum of its components must be positive. If set to
\code{NULL} (the default), all vertices will have the same positive weight
(multiplicity) of 1, i.e., \code{g} is treated as a vertex unweighted graph. The
length of the \code{vertex_weight} must be equivalent to the number of vertices.}

\item{alpha}{A number or a numeric vector of locality levels. Values
must be between 0 and 1.}

\item{edge_weight_transform}{An optional function to transform the edge weights
when \code{g} is an \code{igraph} object and an edge weight attribute exists. This
argument is ignored when \code{g} is a distance matrix.}

\item{x}{An \code{L1centEDGE} object, obtained as a result of the function
\code{L1centEDGE()}.}

\item{...}{Further arguments passed to or from other methods.
\itemize{
\item \code{plot()} method: Further graphical parameters supplied to the internal
\code{\link[igraph:plot.igraph]{igraph::plot.igraph()}} function. See \link[igraph:plot.common]{igraph::plot.common} document.
\item \code{print()} and \code{summary()} methods: Further arguments passed to the
\code{\link[base:print]{base::print()}} and \code{\link[base:summary]{base::summary()}} functions, respectively.
}}

\item{object}{An \code{L1centEDGE} object, obtained as a result of the function
\code{L1centEDGE()}.}
}
\value{
\code{L1centEDGE()} returns an object of class \code{L1centEDGE}. It
is a list of \sQuote{edge lists}. The length of the list is equivalent to
the length of \code{alpha}, and the names of the list are the values of
\code{alpha}. The \code{i}th component of the list is a 2-column matrix,
and each row defines one directed edge, i.e., it is an edge list. The
second column is the local (level \code{alpha[i]}) median of the vertex at
the first column. There may be more than one edge from each vertex, since
there may be more than one local median.

\code{print.L1centEDGE()} prints the edge lists and returns them invisibly.

\code{plot.L1centEDGE()} draws directed graphs corresponding to each value
of \code{alpha}. In each display, each vertex gives a directed edge to its
local median vertex. The local median vertices are shown with larger
circles.

\code{summary.L1centEDGE()} returns an object of class \code{table} with the
number of local medians at each locality level \code{alpha}.
}
\description{
Derives a multiscale edge representation. Each vertex is connected to its
local median, which is found in its
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality-based
neighborhood.
}
\details{
In a global perspective, any given undirected graph can be represented as a
star-shaped directed graph, with each vertex making a connection to the
median vertex. Based on this idea, an undirected graph can be represented as
a directed graph, with each vertex making a connection to the \emph{local}
median vertex. The local median vertex of, say, \eqn{v_i}, is defined as a
median vertex among the
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality-based
neighborhood of \eqn{v_i}. By varying the level of locality, the given graph
can be visually inspected at multiple scales. Refer to Kang and Oh (2026) for
details.
}
\note{
The function is valid only for undirected and connected graphs.
}
\examples{
library(igraph)
MCU_edge <- L1centEDGE(MCUmovie, vertex_weight = V(MCUmovie)$worldwidegross, alpha = 5/32)
plot(MCU_edge)
}
\references{
S. Kang and H.-S. Oh. On a notion of graph centrality based on
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} data depth.
\emph{Journal of the American Statistical Association}, 121(553): 400--412, 2026.
}
\seealso{
\code{\link[=L1cent]{L1cent()}}, \code{\link[=L1centNB]{L1centNB()}}, \code{\link[=L1centLOC]{L1centLOC()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/L1centrality-package.R
\docType{package}
\name{L1centrality-package}
\alias{L1centrality-package}
\alias{L1centrality}
\title{L1centrality: Graph/Network Analysis Based on \out{<i>L</i><sub>1</sub>}
Centrality}
\description{
Analyze graph/network data using \out{<i>L</i><sub>1</sub>} centrality and
prestige. Functions for deriving global, local, and group
\out{<i>L</i><sub>1</sub>} centrality/prestige are provided. Routines for
visual inspection of a graph/network are also provided. Details are in Kang
and Oh (2026a,b) and Kang (2025).
}
\details{
Every function inside this package supports a distinct variety of graphs.
Edge weights can be considered by all functions. Additionally, it is
assumed that the graph to be analyzed is a connected graph in the case of an
undirected graph, or a strongly connected graph in the case of a directed
graph.

Certain functions exclusively accommodate undirected graphs without vertex
multiplicity, while others support both directed and
undirected graphs with vertex multiplicities. The following table provides a
concise overview of the support range for each function.

\tabular{rccc}{
\strong{Functions} \tab \strong{Undirected graph} \tab \strong{Directed graph} \tab \strong{Vertex multiplicity}\cr
\code{\link[=L1cent]{L1cent()}}, \code{\link[=L1centNB]{L1centNB()}}, \code{\link[=L1centLOC]{L1centLOC()}}, \code{\link[=L1centGROUP]{L1centGROUP()}} \tab O \tab O \tab O \cr
\code{\link[=L1centEDGE]{L1centEDGE()}} \tab O \tab X \tab O \cr
\code{\link[=L1centMDS]{L1centMDS()}} \tab O \tab X \tab X
}
}
\references{
S. Kang. \emph{Topics in Non-Euclidean Dimension Reduction}. PhD
thesis, Seoul National University, 2025.

S. Kang and H.-S. Oh. On a notion of graph centrality based on
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} data depth.
\emph{Journal of the American Statistical Association}, 121(553): 400--412, 2026a.

S. Kang and H.-S. Oh.
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} prominence measures
for directed graphs. \emph{The American Statistician}, 80(2): 301--309, 2026b.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/seungwoo-stat/L1centrality}
  \item Report bugs at \url{https://github.com/seungwoo-stat/L1centrality/issues}
}

}
\author{
\strong{Maintainer}: Seungwoo Kang \email{kangsw@skku.edu} (\href{https://orcid.org/0000-0001-8082-0794}{ORCID})

Authors:
\itemize{
  \item Hee-Seok Oh \email{heeseok@stats.snu.ac.kr}
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/L1centGROUP.R
\name{group_reduce}
\alias{group_reduce}
\alias{group_reduce.igraph}
\alias{group_reduce.matrix}
\title{Group Reduced Graph}
\usage{
group_reduce(g, nodes, vertex_weight, method, edge_weight_transform)

\method{group_reduce}{igraph}(
  g,
  nodes,
  vertex_weight = NULL,
  method = c("minimum", "maximum", "average"),
  edge_weight_transform = NULL
)

\method{group_reduce}{matrix}(
  g,
  nodes,
  vertex_weight = NULL,
  method = "minimum",
  edge_weight_transform = NULL
)
}
\arguments{
\item{g}{An \code{igraph} graph object or a distance matrix. When \code{g} is
a distance matrix, the
\ifelse{html}{\out{(<i>i,j</i>)}}{\eqn{(i,j)}} component of the distance
matrix is the geodesic distance from the
\ifelse{html}{\out{<i>i</i>}}{\eqn{i}}th vertex to the
\ifelse{html}{\out{<i>j</i>}}{\eqn{j}}th vertex.}

\item{nodes}{A vector of integers. Each integer indicates the index of the
vertex.}

\item{vertex_weight}{An optional nonnegative multiplicity (weight) vector for (vertex)
weighted networks. The sum of its components must be positive. If set to
\code{NULL} (the default), all vertices will have the same positive weight
(multiplicity) of 1, i.e., \code{g} is treated as a vertex unweighted graph. The
length of the \code{vertex_weight} must be equivalent to the number of vertices.}

\item{method}{A character string. It specifies the method of setting the edge
weight between the pseudo-vertex and the other vertices. Note that the S3
method for the \code{matrix} class only supports the \code{minimum} option. This
is because it is not possible to derive the group reduced graph's distance
matrix from the original distance matrix when using the maximum or average
method. On the other hand, the group reduced graph's distance matrix can be
derived from the original distance matrix when the minimum method is used.
See the discussion in Kang (2025).
\itemize{
\item \code{minimum} (the default): the minimum method is used in setting the edge weights.
\item \code{maximum}: the maximum method is used in setting the edge weights.
\item \code{average}: the average method is used in setting the edge weights.
}}

\item{edge_weight_transform}{An optional function to transform the edge weights
when \code{g} is an \code{igraph} object and an edge weight attribute exists. This
argument is ignored when \code{g} is a distance matrix.}
}
\value{
A list consisting of three objects:
\itemize{
\item \sQuote{distmat}: A matrix representing the group reduced graph's
distance matrix, where the first row and column correspond to the
pseudo-vertex.
\item \sQuote{vertex_weight}: A vector of the group reduced graph's vertex
multiplicity. The first element corresponds to the pseudo-vertex.
\item \sQuote{label}: A vector of the vertex names specified by \code{nodes}.
}
}
\description{
Computes the vertex multiplicities (vertex weights) and the distance matrix of the
group reduced graph. The group reduced graph is constructed by replacing a
group of vertices in the original graph by a single \sQuote{pseudo-vertex}.
}
\details{
The group reduced graph is constructed by replacing the vertices indicated in
the argument \code{nodes} with a single \sQuote{pseudo-vertex}. The
multiplicity (vertex weight) of this new vertex is set to the sum of the
multiplicities of the vertices within \code{nodes}. An edge from the
pseudo-vertex to any vertex that is not in \code{nodes}, say
\ifelse{html}{\out{<i>v</i>}}{\eqn{v}}, is created in the group reduced graph if
there is at least one edge from the vertices in \code{nodes} to
\ifelse{html}{\out{<i>v</i>}}{\eqn{v}} in the original graph. The weight of
this newly added edge is determined using one of the following methods:
\itemize{
\item Minimum method: The edge weight from the pseudo-vertex to
\ifelse{html}{\out{<i>v</i>}}{\eqn{v}} is set to the minimum of the edge
weights of the edges between the vertices in \code{nodes} to
\ifelse{html}{\out{<i>v</i>}}{\eqn{v}} in the original graph.
\item Maximum method: The edge weight from the pseudo-vertex to
\ifelse{html}{\out{<i>v</i>}}{\eqn{v}} is set to the maximum of the edge
weights of the edges between the vertices in \code{nodes} to
\ifelse{html}{\out{<i>v</i>}}{\eqn{v}} in the original graph.
\item Average method: The edge weight from the pseudo-vertex to
\ifelse{html}{\out{<i>v</i>}}{\eqn{v}} is set to the average of the edge
weights of the edges between the vertices in \code{nodes} to
\ifelse{html}{\out{<i>v</i>}}{\eqn{v}} in the original graph.
}
An edge from \ifelse{html}{\out{<i>v</i>}}{\eqn{v}} to the pseudo-vertex is
set in a similar manner. For details, refer to Kang (2025).
}
\note{
Multiple edges (edges with the same head and tail vertices) are not allowed,
because they make the edge weight setting procedure confusing.
}
\examples{
# Group reduced graph of the 'Iron Man' series using the minimum method
vertex_weight <- igraph::V(MCUmovie)$worldwidegross
ironman_series <- c(1,3,7)
reduced_graph <- group_reduce(MCUmovie, nodes = ironman_series, vertex_weight = vertex_weight)
reduced_graph$distmat[1:3,1:3]
reduced_graph$label

# Multiplicity of the pseudo-vertex equals the sums of the replaced vertices' multiplicities
reduced_graph$vertex_weight[1] == sum(vertex_weight[ironman_series])
}
\references{
S. Kang. \emph{Topics in Non-Euclidean Dimension Reduction}. PhD thesis,
Seoul National University, 2025.
}
\seealso{
\code{\link[=L1cent]{L1cent()}} for
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} centrality/prestige.
\code{\link[=L1centGROUP]{L1centGROUP()}} internally uses \code{group_reduce()}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Lorenz_plot.R
\name{Heterogeneity}
\alias{Heterogeneity}
\alias{Lorenz_plot}
\alias{Gini}
\title{Lorenz Curve and the Gini Coefficient}
\usage{
Lorenz_plot(x, add = FALSE, ...)

Gini(x)
}
\arguments{
\item{x}{A numeric vector.}

\item{add}{A logical value.
\itemize{
\item \code{TRUE}: add the Lorenz curve to an already existing plot.
\item \code{FALSE} (the default): draw the Lorenz curve to a new graphic device.
}}

\item{...}{Further graphical parameters supplied to the internal
\code{\link[base:plot]{base::plot()}} (when \code{add = FALSE}) or \code{\link[graphics:lines]{graphics::lines()}} (when
\code{add = TRUE}) function. See \code{\link[graphics:par]{graphics::par()}} document.}
}
\value{
\code{Lorenz_plot()} draws a Lorenz curve (the group heterogeneity plot) and returns an
invisible copy of a Gini coefficient (the group heterogeneity index).

\code{Gini()} returns a Gini coefficient.
}
\description{
Draws a Lorenz curve (the group heterogeneity plot) and computes the Gini
coefficient (the group heterogeneity index).
}
\examples{
vertex_weight <- igraph::V(MCUmovie)$worldwidegross
cent <- L1cent(MCUmovie, vertex_weight = vertex_weight)
gini <- Lorenz_plot(cent, asp = 1) # one can use "plot(cent, asp = 1)"
graphics::abline(0, 1, lty = 2)
# group heterogeneity index
gini
gini == Gini(cent)
}
\references{
S. Kang and H.-S. Oh. On a notion of graph centrality based on
\ifelse{html}{\out{<i>L</i><sub>1</sub>}}{{\eqn{L_1}}} data depth.
\emph{Journal of the American Statistical Association}, 121(553): 400--412, 2026.

M. O. Lorenz. Methods of measuring the concentration of wealth.
\emph{Publications of the American Statistical Association}, 9(70): 209--219, 1905.
}
\seealso{
\code{plot()} methods for objects of class \link{L1cent} and \link{L1centLOC}
support plotting a Lorenz curve. \code{summary()} methods for objects of
class \link{L1cent}, \link{L1centLOC}, and \link{L1centNB} provide the Gini coefficient as
one of the summary statistics.
}
